import sys

# `MD_FLAGS` maps the flag to its bit position.
# The bit positions must match those defined in src/mongoc/mongoc-handshake-private.h
MD_FLAGS = {
    "MONGOC_MD_FLAG_ENABLE_CRYPTO": 0,
    "MONGOC_MD_FLAG_ENABLE_CRYPTO_CNG": 1,
    "MONGOC_MD_FLAG_ENABLE_CRYPTO_COMMON_CRYPTO": 2,
    "MONGOC_MD_FLAG_ENABLE_CRYPTO_LIBCRYPTO": 3,
    "MONGOC_MD_FLAG_ENABLE_CRYPTO_SYSTEM_PROFILE": 4,
    "MONGOC_MD_FLAG_ENABLE_SASL": 5,
    "MONGOC_MD_FLAG_ENABLE_SSL": 6,
    "MONGOC_MD_FLAG_ENABLE_SSL_OPENSSL": 7,
    "MONGOC_MD_FLAG_ENABLE_SSL_SECURE_CHANNEL": 8,
    "MONGOC_MD_FLAG_ENABLE_SSL_SECURE_TRANSPORT": 9,
    "MONGOC_MD_FLAG_EXPERIMENTAL_FEATURES": 10,
    "MONGOC_MD_FLAG_HAVE_SASL_CLIENT_DONE": 11,
    "MONGOC_MD_FLAG_HAVE_WEAK_SYMBOLS": 12,
    "MONGOC_MD_FLAG_NO_AUTOMATIC_GLOBALS": 13,
    "MONGOC_MD_FLAG_ENABLE_SSL_LIBRESSL": 14,
    "MONGOC_MD_FLAG_ENABLE_SASL_CYRUS": 15,
    "MONGOC_MD_FLAG_ENABLE_SASL_SSPI": 16,
    "MONGOC_MD_FLAG_HAVE_SOCKLEN": 17,
    "MONGOC_MD_FLAG_ENABLE_COMPRESSION": 18,
    "MONGOC_MD_FLAG_ENABLE_COMPRESSION_SNAPPY": 19,
    "MONGOC_MD_FLAG_ENABLE_COMPRESSION_ZLIB": 20,
    "MONGOC_MD_FLAG_ENABLE_SASL_GSSAPI": 21,
    "MONGOC_MD_FLAG_ENABLE_RES_NSEARCH": 22,
    "MONGOC_MD_FLAG_ENABLE_RES_NDESTROY": 23,
    "MONGOC_MD_FLAG_ENABLE_RES_NCLOSE": 24,
    "MONGOC_MD_FLAG_ENABLE_RES_SEARCH": 25,
    "MONGOC_MD_FLAG_ENABLE_DNSAPI": 26,
    "MONGOC_MD_FLAG_ENABLE_RDTSCP": 27,
    "MONGOC_MD_FLAG_HAVE_SCHED_GETCPU": 28,
    "MONGOC_MD_FLAG_ENABLE_SHM_COUNTERS": 29,
    "MONGOC_MD_FLAG_TRACE": 30,
    # `MONGOC_MD_FLAG_ENABLE_ICU` was accidentally removed in libmongoc 1.25.0-1.25.3.
    # If parsing a config-bitfield produced by libmongoc 1.25.0-1.25.3, use the version of `parse_handshake_cfg.py` from the git tag 1.25.0.
    "MONGOC_MD_FLAG_ENABLE_ICU": 31,
    "MONGOC_MD_FLAG_ENABLE_CLIENT_SIDE_ENCRYPTION": 32,
    "MONGOC_MD_FLAG_ENABLE_MONGODB_AWS_AUTH": 33,
    "MONGOC_MD_FLAG_ENABLE_SRV": 34,
    "MONGOC_MD_FLAG_HAVE_BCRYPT_PBKDF2": 35,
}

def main():
    flag_to_number = {s: 2 ** i for s,i in MD_FLAGS.items()}

    if len(sys.argv) < 2:
        print ("Usage: python {0} config-bitfield".format(sys.argv[0]))
        print ("Example: python parse_handshake_cfg.py 0x3e65")
        return

    config_bitfield_string = sys.argv[1]
    config_bitfield_num = int(config_bitfield_string, 0)
    print ("Decimal value: {}".format(config_bitfield_num))

    for flag, num in flag_to_number.items():
        v = "true" if config_bitfield_num & num else "false"
        print ("{:<50}: {}".format(flag, v))

if __name__ == "__main__":
    main()



from shrub.v3.evg_build_variant import BuildVariant
from shrub.v3.evg_task import EvgTaskRef

from config_generator.etc.function import merge_defns
from config_generator.etc.compile import generate_compile_tasks

from config_generator.etc.sasl.compile import CompileCommon
from config_generator.etc.sasl.test import generate_test_tasks


SSL = 'darwinssl'
TAG = f'sasl-matrix-{SSL}'


# pylint: disable=line-too-long
# fmt: off
COMPILE_MATRIX = [
    ('macos-14', 'clang', None, ['cyrus']),

    ('macos-11-arm64', 'clang', None, ['cyrus']),
    ('macos-14-arm64', 'clang', None, ['cyrus']),
]

TEST_MATRIX = [
    ('macos-14', 'clang', None, 'cyrus', ['auth'], ['server'], ['4.0', '4.2', '4.4', '5.0']),

    ('macos-11-arm64', 'clang', None, 'cyrus', ['auth'], ['server'], ['6.0', '7.0',                ]),
    ('macos-14-arm64', 'clang', None, 'cyrus', ['auth'], ['server'], ['6.0', '7.0', '8.0', 'latest']),
]
# fmt: on
# pylint: enable=line-too-long


class DarwinSSLCompileCommon(CompileCommon):
    ssl = 'DARWIN'


class SaslCyrusDarwinSSLCompile(DarwinSSLCompileCommon):
    name = 'sasl-cyrus-darwinssl-compile'
    commands = DarwinSSLCompileCommon.compile_commands(sasl='CYRUS')


def functions():
    return merge_defns(
        SaslCyrusDarwinSSLCompile.defn(),
    )


def tasks():
    res = []

    SASL_TO_FUNC = {
        'cyrus': SaslCyrusDarwinSSLCompile,
    }

    res += generate_compile_tasks(SSL, TAG, SASL_TO_FUNC, COMPILE_MATRIX)
    res += generate_test_tasks(SSL, TAG, TEST_MATRIX)

    return res


def variants():
    expansions = {}

    return [
        BuildVariant(
            name=TAG,
            display_name=TAG,
            tasks=[EvgTaskRef(name=f'.{TAG}')],
            expansions=expansions,
        ),
    ]



# Copyright 2009-present MongoDB, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from collections import OrderedDict as OD
from itertools import chain
from typing import ClassVar, Iterable, Literal, Mapping, MutableMapping, MutableSequence, Optional, Sequence

from evergreen_config_generator import Value, Scalar
from evergreen_config_generator.functions import func, s3_put
from evergreen_config_generator.tasks import (
    both_or_neither,
    MatrixTask,
    NamedTask,
    prohibit,
    require,
    Task,
    DependencySpec,
)
from evergreen_config_lib import shell_mongoc
from packaging.version import Version


ToggleStr = Literal["OFF", "ON"]
OptToggleStr = Optional[ToggleStr]
TopologyStr = Literal["server"]


class CompileTask(NamedTask):
    cls_compile_sh_env: ClassVar[Mapping[str, str]] = {}
    cls_tags: ClassVar[Sequence[str]] = ()
    cls_sanitize: ClassVar[Sequence[str]] = ()

    def __init__(
        self,
        task_name: str,
        tags: Iterable[str] = (),
        config: str = "debug",
        compression: str | None = "default",
        suffix_commands: Iterable[Value] = (),
        depends_on: Iterable[DependencySpec] = (),
        prefix_commands: Iterable[Value] = (),
        sanitize: Iterable[Literal["undefined", "address", "thread"]] = (),
        *,
        CFLAGS: str | None = None,
        LDFLAGS: str | None = None,
        EXTRA_CONFIGURE_FLAGS: str | None = None,
        SSL: Literal["WINDOWS", "DARWIN", "OPENSSL", "OPENSSL_STATIC", "LIBRESSL", "OFF", None] = None,
        ENABLE_SHM_COUNTERS: OptToggleStr = None,
        CHECK_LOG: OptToggleStr = None,
        TRACING: OptToggleStr = None,
        SASL: Literal[None, "OFF", "AUTO", "CYRUS", "SSPI"] = None,
        ENABLE_RDTSCP: OptToggleStr = None,
        SRV: OptToggleStr = None,
    ):
        super(CompileTask, self).__init__(task_name=task_name, depends_on=depends_on, tags=tags)

        self.suffix_commands = list(suffix_commands)
        self.prefix_commands = list(prefix_commands)

        # Environment variables for .evergreen/scripts/compile.sh.
        self.compile_sh_opt: dict[str, str] = {}

        if config != "debug":
            assert config == "release"
            self.compile_sh_opt["RELEASE"] = "ON"

        if CFLAGS:
            self.compile_sh_opt["CFLAGS"] = CFLAGS
        if LDFLAGS:
            self.compile_sh_opt["LDFLAGS"] = LDFLAGS
        if EXTRA_CONFIGURE_FLAGS:
            self.compile_sh_opt["EXTRA_CONFIGURE_FLAGS"] = EXTRA_CONFIGURE_FLAGS
        if SSL:
            self.compile_sh_opt["SSL"] = SSL
        if ENABLE_SHM_COUNTERS:
            self.compile_sh_opt["ENABLE_SHM_COUNTERS"] = ENABLE_SHM_COUNTERS
        if CHECK_LOG:
            self.compile_sh_opt["CHECK_LOG"] = CHECK_LOG
        if TRACING:
            self.compile_sh_opt["TRACING"] = TRACING
        if SASL:
            self.compile_sh_opt["SASL"] = SASL
        if ENABLE_RDTSCP:
            self.compile_sh_opt["ENABLE_RDTSCP"] = ENABLE_RDTSCP
        if SRV:
            self.compile_sh_opt["SRV"] = SRV

        if compression != "default":
            self.compile_sh_opt["SNAPPY"] = "ON" if compression in ("all", "snappy") else "OFF"
            self.compile_sh_opt["ZLIB"] = "BUNDLED" if compression in ("all", "zlib") else "OFF"
            self.compile_sh_opt["ZSTD"] = "ON" if compression in ("all", "zstd") else "OFF"

        if sanitize:
            self.compile_sh_opt["SANITIZE"] = ",".join(sanitize)

        self.compile_sh_opt.update(type(self).cls_compile_sh_env)

    def additional_script_env(self) -> Mapping[str, str]:
        return {}

    def to_dict(self):
        task = super(CompileTask, self).to_dict()
        commands = task["commands"]
        assert isinstance(commands, MutableSequence), task

        commands.extend(self.prefix_commands)

        script = "env"
        for opt, value in sorted(self.compile_sh_opt.items()):
            script += ' %s="%s"' % (opt, value)

        script += " .evergreen/scripts/compile.sh"

        commands.append(func('find-cmake-latest'))
        commands.append(shell_mongoc(script, add_expansions_to_env=True))
        commands.append(func("upload-build"))
        commands.extend(self.suffix_commands)
        return task

    def additional_tags(self) -> Iterable[str]:
        yield from super().additional_tags()
        yield from self.cls_tags


class SpecialTask(CompileTask):
    cls_tags: ClassVar[Sequence[str]] = ["special"]


class CompileWithClientSideEncryption(CompileTask):
    cls_compile_sh_env: ClassVar[Mapping[str, str]] = dict(
        # Compiling with ClientSideEncryption support requires linking against the library libmongocrypt.
        COMPILE_LIBMONGOCRYPT="ON",
        EXTRA_CONFIGURE_FLAGS="-DENABLE_PIC=ON",
    )
    cls_tags: ClassVar[Sequence[str]] = "client-side-encryption", "special"


class CompileWithClientSideEncryptionAsan(CompileTask):
    cls_compile_sh_env: ClassVar[Mapping[str, str]] = dict(
        CFLAGS="-fno-omit-frame-pointer",
        COMPILE_LIBMONGOCRYPT="ON",
        CHECK_LOG="ON",
        EXTRA_CONFIGURE_FLAGS="-DENABLE_EXTRA_ALIGNMENT=OFF",
        PATH="/usr/lib/llvm-3.8/bin:$PATH",
    )
    cls_tags: ClassVar[Sequence[str]] = ["client-side-encryption"]
    cls_sanitize: ClassVar[Sequence[str]] = ["address"]


class LinkTask(NamedTask):
    def __init__(
        self, task_name: str, suffix_commands: Iterable[Value], orchestration: Literal[True, False, "ssl"] = True
    ):
        if orchestration == "ssl":
            # Actual value of SSL does not matter here so long as it is not 'nossl'.
            bootstrap_commands = [func("fetch-det"), func("bootstrap-mongo-orchestration", SSL="openssl")]
        elif orchestration:
            bootstrap_commands = [func("fetch-det"), func("bootstrap-mongo-orchestration")]
        else:
            bootstrap_commands = []

        super().__init__(
            task_name=task_name,
            commands=bootstrap_commands + list(suffix_commands),
        )


all_tasks = [
    CompileTask(
        "hardened-compile",
        tags=["hardened"],
        compression=None,
        CFLAGS="-fno-strict-overflow -D_FORTIFY_SOURCE=2 -fstack-protector-all -fPIE -O",
        LDFLAGS="-pie -Wl,-z,relro -Wl,-z,now",
    ),
    CompileTask("debug-compile-compression-zlib", tags=["zlib", "compression"], compression="zlib"),
    CompileTask("debug-compile-compression-snappy", tags=["snappy", "compression"], compression="snappy"),
    CompileTask("debug-compile-compression-zstd", tags=["zstd", "compression"], compression="zstd"),
    CompileTask(
        "debug-compile-no-align",
        tags=["debug-compile"],
        compression="zlib",
        EXTRA_CONFIGURE_FLAGS="-DENABLE_EXTRA_ALIGNMENT=OFF",
    ),
    CompileTask("debug-compile-nosasl-nossl", tags=["debug-compile", "nosasl", "nossl"], SSL="OFF"),
    CompileTask("debug-compile-lto", CFLAGS="-flto"),
    CompileTask("debug-compile-lto-thin", CFLAGS="-flto=thin"),
    CompileTask("debug-compile-no-counters", tags=["debug-compile", "no-counters"], ENABLE_SHM_COUNTERS="OFF"),
    CompileTask("compile-tracing", TRACING="ON", CFLAGS="-Werror -Wno-cast-align"),
    CompileTask("release-compile", config="release"),
    CompileTask("debug-compile-nosasl-openssl", tags=["debug-compile", "nosasl", "openssl"], SSL="OPENSSL"),
    CompileTask(
        "debug-compile-nosasl-openssl-static", tags=["debug-compile", "nosasl", "openssl-static"], SSL="OPENSSL_STATIC"
    ),
    CompileTask("debug-compile-nosasl-darwinssl", tags=["debug-compile", "nosasl", "darwinssl"], SSL="DARWIN"),
    CompileTask("debug-compile-nosasl-winssl", tags=["debug-compile", "nosasl", "winssl"], SSL="WINDOWS"),
    CompileTask("debug-compile-sasl-openssl", tags=["debug-compile", "sasl", "openssl"], SASL="AUTO", SSL="OPENSSL"),
    CompileTask(
        "debug-compile-sasl-openssl-static",
        tags=["debug-compile", "sasl", "openssl-static"],
        SASL="AUTO",
        SSL="OPENSSL_STATIC",
    ),
    CompileTask("debug-compile-sasl-darwinssl", tags=["debug-compile", "sasl", "darwinssl"], SASL="AUTO", SSL="DARWIN"),
    CompileTask("debug-compile-rdtscp", ENABLE_RDTSCP="ON"),
    CompileTask("debug-compile-sspi-winssl", tags=["debug-compile", "sspi", "winssl"], SASL="SSPI", SSL="WINDOWS"),
    CompileTask("debug-compile-nosrv", tags=["debug-compile"], SRV="OFF"),
    LinkTask("link-with-cmake", suffix_commands=[func("link sample program", BUILD_SAMPLE_WITH_CMAKE=1)]),
    LinkTask(
        "link-with-cmake-ssl",
        suffix_commands=[func("link sample program", BUILD_SAMPLE_WITH_CMAKE=1, ENABLE_SSL=1)],
    ),
    LinkTask(
        "link-with-cmake-snappy",
        suffix_commands=[func("link sample program", BUILD_SAMPLE_WITH_CMAKE=1, ENABLE_SNAPPY="ON")],
    ),
    LinkTask("link-with-cmake-mac", suffix_commands=[func("link sample program", BUILD_SAMPLE_WITH_CMAKE=1)]),
    LinkTask(
        "link-with-cmake-deprecated",
        suffix_commands=[func("link sample program", BUILD_SAMPLE_WITH_CMAKE=1, BUILD_SAMPLE_WITH_CMAKE_DEPRECATED=1)],
    ),
    LinkTask(
        "link-with-cmake-ssl-deprecated",
        suffix_commands=[
            func(
                "link sample program",
                BUILD_SAMPLE_WITH_CMAKE=1,
                BUILD_SAMPLE_WITH_CMAKE_DEPRECATED=1,
                ENABLE_SSL=1,
            )
        ],
    ),
    LinkTask(
        "link-with-cmake-snappy-deprecated",
        suffix_commands=[
            func(
                "link sample program",
                BUILD_SAMPLE_WITH_CMAKE=1,
                BUILD_SAMPLE_WITH_CMAKE_DEPRECATED=1,
                ENABLE_SNAPPY="ON",
            )
        ],
    ),
    LinkTask(
        "link-with-cmake-mac-deprecated",
        suffix_commands=[func("link sample program", BUILD_SAMPLE_WITH_CMAKE=1, BUILD_SAMPLE_WITH_CMAKE_DEPRECATED=1)],
    ),
    LinkTask("link-with-cmake-windows", suffix_commands=[func("link sample program MSVC")]),
    LinkTask(
        "link-with-cmake-windows-ssl",
        suffix_commands=[func("link sample program MSVC", ENABLE_SSL=1)],
        orchestration="ssl",
    ),
    LinkTask("link-with-cmake-windows-snappy", suffix_commands=[func("link sample program MSVC", ENABLE_SNAPPY="ON")]),
    LinkTask("link-with-cmake-mingw", suffix_commands=[func("link sample program mingw")]),
    LinkTask("link-with-pkg-config", suffix_commands=[func("link sample program")]),
    LinkTask("link-with-pkg-config-mac", suffix_commands=[func("link sample program")]),
    LinkTask("link-with-pkg-config-ssl", suffix_commands=[func("link sample program", ENABLE_SSL=1)]),
    LinkTask("link-with-bson", suffix_commands=[func("link sample program bson")], orchestration=False),
    LinkTask("link-with-bson-mac", suffix_commands=[func("link sample program bson")], orchestration=False),
    LinkTask("link-with-bson-windows", suffix_commands=[func("link sample program MSVC bson")], orchestration=False),
    LinkTask("link-with-bson-mingw", suffix_commands=[func("link sample program mingw bson")], orchestration=False),
    NamedTask(
        "debian-package-build",
        commands=[
            shell_mongoc('export IS_PATCH="${is_patch}"\n' ".evergreen/scripts/debian_package_build.sh"),
            s3_put(
                local_file="deb.tar.gz",
                remote_file="${branch_name}/mongo-c-driver-debian-packages-${CURRENT_VERSION}.tar.gz",
                content_type="${content_type|application/x-gzip}",
            ),
            s3_put(
                local_file="deb.tar.gz",
                remote_file="${branch_name}/${revision}/${version_id}/${build_id}/${execution}/mongo-c-driver-debian-packages.tar.gz",
                content_type="${content_type|application/x-gzip}",
            ),
            s3_put(
                local_file="deb-i386.tar.gz",
                remote_file="${branch_name}/mongo-c-driver-debian-packages-i386-${CURRENT_VERSION}.tar.gz",
                content_type="${content_type|application/x-gzip}",
            ),
            s3_put(
                local_file="deb-i386.tar.gz",
                remote_file="${branch_name}/${revision}/${version_id}/${build_id}/${execution}/mongo-c-driver-debian-packages-i386.tar.gz",
                content_type="${content_type|application/x-gzip}",
            ),
        ],
    ),
    NamedTask(
        "rpm-package-build",
        commands=[
            shell_mongoc('export IS_PATCH="${is_patch}"\n' ".evergreen/scripts/check_rpm_spec.sh"),
            shell_mongoc(".evergreen/scripts/build_snapshot_rpm.sh"),
            s3_put(
                local_file="rpm.tar.gz",
                remote_file="${branch_name}/mongo-c-driver-rpm-packages-${CURRENT_VERSION}.tar.gz",
                content_type="${content_type|application/x-gzip}",
            ),
            s3_put(
                local_file="rpm.tar.gz",
                remote_file="${branch_name}/${revision}/${version_id}/${build_id}/${execution}/mongo-c-driver-rpm-packages.tar.gz",
                content_type="${content_type|application/x-gzip}",
            ),
            shell_mongoc("sudo rm -rf ../build ../mock-result ../rpm.tar.gz\n" "export MOCK_TARGET_CONFIG=rocky+epel-9-aarch64\n" ".evergreen/scripts/build_snapshot_rpm.sh"),
            shell_mongoc("sudo rm -rf ../build ../mock-result ../rpm.tar.gz\n" "export MOCK_TARGET_CONFIG=rocky+epel-8-aarch64\n" ".evergreen/scripts/build_snapshot_rpm.sh"),
        ],
    ),
    NamedTask(
        "install-uninstall-check-mingw",
        commands=[
            func("find-cmake-latest"),
            shell_mongoc(
                r"""
                . .evergreen/scripts/find-cmake-latest.sh
                export CMAKE="$(find_cmake_latest)"
                export CC="C:/mingw-w64/x86_64-4.9.1-posix-seh-rt_v3-rev1/mingw64/bin/gcc.exe"
                BSON_ONLY=1 cmd.exe /c .\\.evergreen\\scripts\\install-uninstall-check-windows.cmd
                cmd.exe /c .\\.evergreen\\scripts\\install-uninstall-check-windows.cmd""",
                include_expansions_in_env=["distro_id"],
            )
        ],
    ),
    NamedTask(
        "install-uninstall-check-msvc",
        commands=[
            func("find-cmake-latest"),
            shell_mongoc(
                r"""
                . .evergreen/scripts/find-cmake-latest.sh
                export CMAKE="$(find_cmake_latest)"
                export CC="Visual Studio 14 2015 Win64"
                BSON_ONLY=1 cmd.exe /c .\\.evergreen\\scripts\\install-uninstall-check-windows.cmd
                cmd.exe /c .\\.evergreen\\scripts\\install-uninstall-check-windows.cmd""",
                include_expansions_in_env=["distro_id"],
            )
        ],
    ),
    NamedTask(
        "install-uninstall-check",
        commands=[
            func("find-cmake-latest"),
            shell_mongoc(
                r"""
                . .evergreen/scripts/find-cmake-latest.sh
                export CMAKE="$(find_cmake_latest)"
                DESTDIR="$(pwd)/dest" .evergreen/scripts/install-uninstall-check.sh
                BSON_ONLY=1 .evergreen/scripts/install-uninstall-check.sh
                .evergreen/scripts/install-uninstall-check.sh""",
                include_expansions_in_env=["distro_id"],
            )
        ],
    ),
    CompileTask("debug-compile-with-warnings", CFLAGS="-Werror -Wno-cast-align"),
    CompileTask(
        "debug-compile-nosasl-openssl-1.0.1",
        prefix_commands=[func("install ssl", SSL="openssl-1.0.1u")],
        CFLAGS="-Wno-redundant-decls",
        SSL="OPENSSL",
        SASL="OFF",
    ),
    NamedTask(
        "build-and-test-with-toolchain",
        commands=[
            OD(
                [
                    ("command", "s3.get"),
                    (
                        "params",
                        OD(
                            [
                                ("aws_key", "${aws_key}"),
                                ("aws_secret", "${aws_secret}"),
                                ("remote_file", "mongo-c-toolchain/${distro_id}/2023/06/07/mongo-c-toolchain.tar.gz"),
                                ("bucket", "mongo-c-toolchain"),
                                ("local_file", "mongo-c-toolchain.tar.gz"),
                            ]
                        ),
                    ),
                ]
            ),
            shell_mongoc(".evergreen/scripts/build-and-test-with-toolchain.sh"),
        ],
    ),
    NamedTask(
        "install-libmongoc-after-libbson",
        commands=[shell_mongoc(".evergreen/scripts/install-libmongoc-after-libbson.sh"),],
    ),
]


class CoverageTask(MatrixTask):
    axes = OD(
        [
            ("version", ["latest"]),
            ("topology", ["replica_set"]),
            ("auth", [True]),
            ("sasl", ["sasl"]),
            ("ssl", ["openssl"]),
            ("cse", [False, True]),
        ]
    )

    def additional_tags(self) -> Iterable[str]:
        yield from super().additional_tags()
        yield "test-coverage"
        yield str(self.settings.version)
        if self.cse:
            yield "client-side-encryption"

    def name_parts(self) -> Iterable[str]:
        yield "test-coverage"
        yield self.display("version")
        yield self.display("topology").replace("_", "-")
        yield from map(self.display, ("auth", "sasl", "ssl"))
        if self.settings.cse:
            yield "cse"

    @property
    def cse(self) -> bool:
        return bool(self.settings.cse)

    def post_commands(self) -> Iterable[Value]:
        if self.cse:
            yield func(
                "compile coverage",
                SASL="AUTO",
                SSL="OPENSSL",
                COMPILE_LIBMONGOCRYPT="ON",
                EXTRA_CONFIGURE_FLAGS='EXTRA_CONFIGURE_FLAGS="-DENABLE_PIC=ON"',
            )
        else:
            yield func("compile coverage", SASL="AUTO", SSL="OPENSSL")

        yield func("fetch-det")
        yield func(
            "bootstrap-mongo-orchestration",
            MONGODB_VERSION=self.settings.version,
            TOPOLOGY=self.settings.topology,
            AUTH=self.display("auth"),
            SSL=self.display("ssl"),
        )
        yield func("run-simple-http-server")
        extra = {"COVERAGE": "ON"}
        if self.cse:
            extra["CLIENT_SIDE_ENCRYPTION"] = "ON"
            yield func("run-mock-kms-servers")
        yield func("run-tests", AUTH=self.display("auth"), SSL=self.display("ssl"), **extra)
        yield func("upload coverage")
        yield func("update codecov.io")

    def do_is_valid_combination(self) -> bool:
        # Limit coverage tests to test-coverage-latest-replica-set-auth-sasl-openssl (+ cse).
        require(self.setting_eq("topology", "replica_set"))
        require(self.setting_eq("sasl", "sasl"))
        require(self.setting_eq("ssl", "openssl"))
        require(self.setting_eq("version", "latest"))
        require(self.settings.auth is True)

        if not self.cse:
            # No further requirements
            return True

        # CSE has extra requirements
        if self.settings.version != "latest":
            # We only work with 4.2 or newer for CSE
            require(Version(str(self.settings.version)) >= Version("4.2"))
        return True


all_tasks = chain(all_tasks, CoverageTask.matrix())


class DNSTask(MatrixTask):
    axes = OD(
        [
            ("auth", [False, True]),
            ("loadbalanced", [False, True]),
            ("ssl", ["openssl", "winssl", "darwinssl"]),
        ]
    )

    name_prefix = "test-dns"

    def additional_dependencies(self) -> Iterable[DependencySpec]:
        yield self.build_task_name

    @property
    def build_task_name(self) -> str:
        sasl = "sspi" if self.settings.ssl == "winssl" else "sasl"
        return f'debug-compile-{sasl}-{self.display("ssl")}'

    def name_parts(self) -> Iterable[str]:
        yield "test-dns"
        if self.settings.auth:
            yield "auth"
        if self.settings.loadbalanced:
            yield "loadbalanced"
        yield self.display("ssl")

    def post_commands(self) -> Iterable[Value]:
        yield func("fetch-build", BUILD_NAME=self.build_task_name)
        yield func("fetch-det")
        if self.settings.loadbalanced:
            orchestration = func(
                "bootstrap-mongo-orchestration",
                TOPOLOGY="sharded_cluster",
                AUTH="auth" if self.settings.auth else "noauth",
                SSL="ssl",
                LOAD_BALANCER="on",
            )
        else:
            orchestration = func(
                "bootstrap-mongo-orchestration",
                TOPOLOGY="replica_set",
                AUTH="auth" if self.settings.auth else "noauth",
                SSL="ssl",
            )

        yield orchestration

        dns = "on"
        if self.settings.loadbalanced:
            dns = "loadbalanced"
            yield func("fetch-det")
            yield func("start-load-balancer", MONGODB_URI="mongodb://localhost:27017,localhost:27018")
        elif self.settings.auth:
            dns = "dns-auth"
        yield func("run-tests", SSL="ssl", AUTH=self.display("auth"), DNS=dns)

    def do_is_valid_combination(self) -> bool:
        prohibit(bool(self.settings.loadbalanced) and bool(self.settings.auth))
        # Load balancer tests only run on some Linux hosts in Evergreen until CDRIVER-4041 is resolved.
        prohibit(bool(self.settings.loadbalanced) and self.settings.ssl in ["darwinssl", "winssl"])
        return True


all_tasks = chain(all_tasks, DNSTask.matrix())


class CompressionTask(MatrixTask):
    axes = OD([("compression", ["zlib", "snappy", "zstd"])])
    name_prefix = "test-latest-server"

    def additional_dependencies(self) -> Iterable[DependencySpec]:
        yield self.build_task_name

    @property
    def build_task_name(self) -> str:
        return f"debug-compile-{self._compressor_suffix()}"

    def additional_tags(self) -> Iterable[str]:
        yield from super().additional_tags()
        yield "compression"
        yield "latest"
        yield from self._compressor_list()

    def name_parts(self) -> Iterable[str]:
        return [self.name_prefix, self._compressor_suffix()]

    def post_commands(self) -> Iterable[Value]:
        yield func("fetch-build", BUILD_NAME=self.build_task_name)
        yield func("fetch-det")
        yield func("bootstrap-mongo-orchestration", AUTH="noauth", SSL="nossl")
        yield func("run-simple-http-server")
        yield func("run-tests", AUTH="noauth", SSL="nossl", COMPRESSORS=",".join(self._compressor_list()))

    def _compressor_suffix(self):
        if self.settings.compression == "zlib":
            return "compression-zlib"
        elif self.settings.compression == "snappy":
            return "compression-snappy"
        elif self.settings.compression == "zstd":
            return "compression-zstd"
        else:
            return "compression"

    def _compressor_list(self):
        if self.settings.compression == "zlib":
            return ["zlib"]
        elif self.settings.compression == "snappy":
            return ["snappy"]
        elif self.settings.compression == "zstd":
            return ["zstd"]
        else:
            return ["snappy", "zlib", "zstd"]


all_tasks = chain(all_tasks, CompressionTask.matrix())


class SpecialIntegrationTask(NamedTask):
    def __init__(
        self,
        task_name: str,
        main_dep: str = "debug-compile-sasl-openssl",
        uri: str | None = None,
        tags: Iterable[str] = (),
        version: str = "latest",
        topology: str = "server",
    ):
        self._main_dep = main_dep
        super().__init__(task_name, depends_on=[self._main_dep], tags=tags)
        self._uri = uri
        self._version = version
        self._topo = topology

    def pre_commands(self) -> Iterable[Value]:
        yield func("fetch-build", BUILD_NAME=self._main_dep)
        yield func("fetch-det")
        yield func("bootstrap-mongo-orchestration", MONGODB_VERSION=self._version, TOPOLOGY=self._topo)
        yield func("run-simple-http-server")
        yield func("run-tests", URI=self._uri)


all_tasks = chain(
    all_tasks,
    [
        # Verify that retryWrites=true is ignored with standalone.
        SpecialIntegrationTask("retry-true-latest-server", uri="mongodb://localhost/?retryWrites=true"),
        SpecialIntegrationTask("test-latest-server-hardened", "hardened-compile", tags=["hardened", "latest"]),
    ],
)


class AuthTask(MatrixTask):
    axes = OD([("sasl", ["sasl", "sspi", False]), ("ssl", ["openssl", "darwinssl", "winssl"])])

    name_prefix = "authentication-tests"

    def additional_tags(self) -> Iterable[str]:
        yield from super().additional_tags()
        yield "authentication-tests"
        yield self.display("ssl")
        yield self.display("sasl")

    def additional_dependencies(self) -> Iterable[DependencySpec]:
        yield self.build_task_name

    def post_commands(self) -> Iterable[Value]:
        yield func("fetch-build", BUILD_NAME=self.build_task_name)
        yield func("run auth tests")

    @property
    def build_task_name(self) -> str:
        return f'debug-compile-{self.display("sasl")}-{self.display("ssl")}'

    def name_parts(self) -> Iterable[str]:
        yield self.name_prefix
        yield self.display("ssl")
        if not self.settings.sasl:
            yield "nosasl"

    def do_is_valid_combination(self) -> bool:
        both_or_neither(self.settings.ssl == "winssl", self.settings.sasl == "sspi")
        if not self.settings.sasl:
            require(self.settings.ssl == "openssl")
        return True


all_tasks = chain(all_tasks, AuthTask.matrix())


class PostCompileTask(NamedTask):
    def __init__(self, name: str, tags: Iterable[str], get_build: str, commands: Iterable[Value]):
        super().__init__(name, commands=commands, tags=tags, depends_on=[get_build])
        self._dep = get_build

    def pre_commands(self) -> Iterable[Value]:
        yield func("fetch-build", BUILD_NAME=self._dep)


all_tasks = chain(
    all_tasks,
    [
        PostCompileTask(
            "test-mongohouse",
            tags=[],
            get_build="debug-compile-sasl-openssl",
            commands=[func("fetch-det"), func("build mongohouse"), func("run mongohouse"), func("test mongohouse")],
        ),
        NamedTask(
            "authentication-tests-asan-memcheck",
            tags=["authentication-tests", "asan"],
            commands=[
                func("find-cmake-latest"),
                shell_mongoc(
                    """
            env SANITIZE=address SASL=AUTO SSL=OPENSSL EXTRA_CONFIGURE_FLAGS='-DENABLE_EXTRA_ALIGNMENT=OFF' .evergreen/scripts/compile.sh
            """,
                    add_expansions_to_env=True,
                ),
                func("run auth tests", ASAN="on"),
            ],
        )
    ],
)

# Add API version tasks.
for server_version in [ "8.0", "7.0", "6.0", "5.0"]:
    all_tasks = chain(
        all_tasks,
        [
            PostCompileTask(
                "test-versioned-api-" + server_version,
                tags=["versioned-api", f"{server_version}"],
                get_build="debug-compile-nosasl-openssl",
                commands=[
                    func("fetch-det"),
                    func(
                        "bootstrap-mongo-orchestration",
                        TOPOLOGY="server",
                        AUTH="auth",
                        SSL="ssl",
                        MONGODB_VERSION=server_version,
                        REQUIRE_API_VERSION="true",
                    ),
                    func("run-simple-http-server"),
                    func("run-tests", MONGODB_API_VERSION=1, AUTH="auth", SSL="ssl"),
                ],
            ),
            PostCompileTask(
                "test-versioned-api-accept-version-two-" + server_version,
                tags=["versioned-api", f"{server_version}"],
                get_build="debug-compile-nosasl-nossl",
                commands=[
                    func("fetch-det"),
                    func(
                        "bootstrap-mongo-orchestration",
                        TOPOLOGY="server",
                        AUTH="noauth",
                        SSL="nossl",
                        MONGODB_VERSION=server_version,
                        ORCHESTRATION_FILE="versioned-api-testing.json",
                    ),
                    func("run-simple-http-server"),
                    func("run-tests", MONGODB_API_VERSION=1, AUTH="noauth", SSL="nossl"),
                ],
            )
        ]
    )


class SSLTask(Task):
    def __init__(
        self,
        version: str,
        patch: str,
        cflags: str = "",
        fips: bool = False,
        enable_ssl: str | Literal[False] = False,
        test_params: Mapping[str, Scalar] | None = None,
    ):
        full_version = version + patch + ("-fips" if fips else "")
        self.enable_ssl = enable_ssl
        script = "env"
        if cflags:
            script += f" CFLAGS={cflags}"

        script += " SASL=OFF"

        if enable_ssl is not False:
            script += " SSL=" + enable_ssl
        elif "libressl" in version:
            script += " SSL=LIBRESSL"
        else:
            script += " SSL=OPENSSL"

        script += " .evergreen/scripts/compile.sh"

        super(SSLTask, self).__init__(
            commands=[
                func("install ssl", SSL=full_version),
                func("find-cmake-latest"),
                shell_mongoc(script, add_expansions_to_env=True),
                func("run auth tests", **(test_params or {})),
                func("upload-build"),
            ]
        )

        self.version = version
        self.fips = fips

    @property
    def name(self):
        s = "build-and-run-authentication-tests-" + self.version
        if self.fips:
            return s + "-fips"
        if self.enable_ssl is not False:
            return s + "-" + str(self.enable_ssl).lower()

        return s


all_tasks = chain(
    all_tasks,
    [
        SSLTask(
            "openssl-1.0.1",
            "u",
            cflags="-Wno-redundant-decls",
        ),
        SSLTask("openssl-1.0.1", "u", cflags="-Wno-redundant-decls", fips=True),
        SSLTask(
            "openssl-1.0.2",
            "l",
            cflags="-Wno-redundant-decls",
        ),
        SSLTask("openssl-1.1.0", "l"),
        SSLTask("libressl-2.5", ".2", test_params=dict(require_tls12=True)),
        SSLTask("libressl-3.0", ".2", enable_ssl="AUTO", test_params=dict(require_tls12=True)),
        SSLTask("libressl-3.0", ".2", test_params=dict(require_tls12=True)),
    ],
)


class IPTask(MatrixTask):
    axes = OD(
        [
            ("client", ["ipv6", "ipv4", "localhost"]),
            ("server", ["ipv6", "ipv4"]),
        ]
    )

    name_prefix = "test-latest"

    def additional_dependencies(self) -> Iterable[DependencySpec]:
        yield "debug-compile-nosasl-nossl"

    def additional_tags(self) -> Iterable[str]:
        yield from super().additional_tags()
        yield from ("nossl", "nosasl", "server", "ipv4-ipv6", "latest")

    def post_commands(self) -> Iterable[Value]:
        return [
            func("fetch-build", BUILD_NAME="debug-compile-nosasl-nossl"),
            func("fetch-det"),
            func("bootstrap-mongo-orchestration"),
            func("run-simple-http-server"),
            func(
                "run-tests",
                URI={
                    "ipv6": "mongodb://[::1]/",
                    "ipv4": "mongodb://127.0.0.1/",
                    "localhost": "mongodb://localhost/",
                }[str(self.settings.client)],
            ),
        ]

    def name_parts(self) -> Iterable[str]:
        return (
            self.name_prefix,
            f'server-{self.display("server")}',
            f'client-{self.display("client")}',
            "noauth",
            "nosasl",
            "nossl",
        )

    def do_is_valid_combination(self) -> bool:
        # This would fail by design.
        if self.settings.server == "ipv4":
            prohibit(self.settings.client == "ipv6")

        # Default configuration is tested in other variants.
        if self.settings.server == "ipv6":
            prohibit(self.settings.client == "localhost")
        return True


all_tasks = chain(all_tasks, IPTask.matrix())

aws_compile_task = NamedTask(
    "debug-compile-aws",
    commands=[
        func('find-cmake-latest'),
        shell_mongoc(
            """
            export distro_id='${distro_id}' # Required by find_cmake_latest.
            . .evergreen/scripts/find-cmake-latest.sh
            cmake_binary="$(find_cmake_latest)"

            # Use ccache if able.
            . .evergreen/scripts/find-ccache.sh
            find_ccache_and_export_vars "$(pwd)" || true

            # Compile test-awsauth. Disable unnecessary dependencies since test-awsauth is copied to a remote Ubuntu 20.04 ECS cluster for testing, which may not have all dependent libraries.
            export CC='${CC}'
            "$cmake_binary" -DENABLE_TRACING=ON -DENABLE_SASL=OFF -DENABLE_SNAPPY=OFF -DENABLE_ZSTD=OFF -DENABLE_CLIENT_SIDE_ENCRYPTION=OFF .
            "$cmake_binary" --build . --target test-awsauth
            """
        ),
        func("upload-build"),
    ],
)

all_tasks = chain(all_tasks, [aws_compile_task])


class AWSTestTask(MatrixTask):
    axes = OD(
        [
            ("testcase", ["regular", "ec2", "ecs", "lambda", "assume_role", "assume_role_with_web_identity"]),
            ("version", ["latest", "8.0", "7.0", "6.0", "5.0", "4.4"]),
        ]
    )

    name_prefix = "test-aws-openssl"

    def additional_dependencies(self) -> Iterable[DependencySpec]:
        yield "debug-compile-aws"

    def additional_tags(self) -> Iterable[str]:
        yield from super().additional_tags()
        yield f'{self.settings.version}'
        yield f'test-aws'

    def post_commands(self) -> Iterable[Value]:
        return [
            func("fetch-build", BUILD_NAME="debug-compile-aws"),
            func("fetch-det"),
            func(
                "bootstrap-mongo-orchestration",
                AUTH="auth",
                ORCHESTRATION_FILE="auth-aws.json",
                MONGODB_VERSION=self.settings.version,
                TOPOLOGY="server",
            ),
            func("run aws tests", TESTCASE=str(self.settings.testcase).upper()),
        ]

    @property
    def name(self):
        return f"{self.name_prefix}-{self.settings.testcase}-{self.settings.version}"


all_tasks = chain(all_tasks, AWSTestTask.matrix())


class OCSPTask(MatrixTask):
    axes = OD(
        [
            (
                "test",
                [
                    "test_1",
                    "test_2",
                    "test_3",
                    "test_4",
                    "soft_fail_test",
                    "malicious_server_test_1",
                    "malicious_server_test_2",
                    "cache",
                ],
            ),
            ("delegate", ["delegate", "nodelegate"]),
            ("cert", ["rsa", "ecdsa"]),
            ("ssl", ["openssl", "openssl-1.0.1", "darwinssl", "winssl"]),
            ("version", ["latest", "8.0", "7.0", "6.0", "5.0", "4.4"]),
        ]
    )

    name_prefix = "test-ocsp"

    @property
    def build_task_name(self) -> str:
        return f'debug-compile-nosasl-{self.display("ssl")}'

    def additional_tags(self) -> Iterable[str]:
        yield from super().additional_tags()
        yield f'ocsp-{self.display("ssl")}'

    def additional_dependencies(self) -> Iterable[DependencySpec]:
        yield self.build_task_name

    @property
    def name(self):
        return f"ocsp-{self.settings.ssl}-{self.test}-{self.settings.cert}-{self.settings.delegate}-{self.settings.version}"

    @property
    def test(self) -> str:
        return str(self.settings.test)

    def post_commands(self) -> Iterable[Value]:
        yield func("fetch-build", BUILD_NAME=self.build_task_name)
        yield func("fetch-det")

        stapling = "mustStaple"
        if self.test in ["test_3", "test_4", "soft_fail_test", "cache"]:
            stapling = "disableStapling"
        if self.test in ["malicious_server_test_1", "malicious_server_test_2"]:
            stapling = "mustStaple-disableStapling"

        orchestration = func(
            "bootstrap-mongo-orchestration",
            MONGODB_VERSION=self.settings.version,
            TOPOLOGY="server",
            SSL="ssl",
            OCSP="on",
            ORCHESTRATION_FILE=f"{self.settings.cert}-basic-tls-ocsp-{stapling}.json",
        )

        # The cache test expects a revoked response from an OCSP responder, exactly like TEST_4.
        test_column = "TEST_4" if self.test == "cache" else str(self.test).upper()
        use_delegate = "ON" if self.settings.delegate == "delegate" else "OFF"

        yield (
            shell_mongoc(
                f"""
                TEST_COLUMN={test_column} CERT_TYPE={self.settings.cert} USE_DELEGATE={use_delegate} .evergreen/scripts/run-ocsp-responder.sh
                """
            )
        )

        yield (orchestration)

        if self.build_task_name == "debug-compile-nosasl-openssl-1.0.1":
            # LD_LIBRARY_PATH is needed so the in-tree OpenSSL 1.0.1 is found at runtime
            if self.test == "cache":
                yield (
                    shell_mongoc(
                        f"""
                        LD_LIBRARY_PATH=$(pwd)/install-dir/lib CERT_TYPE={self.settings.cert} .evergreen/scripts/run-ocsp-cache-test.sh
                        """
                    )
                )
            else:
                yield (
                    shell_mongoc(
                        f"""
                        LD_LIBRARY_PATH=$(pwd)/install-dir/lib TEST_COLUMN={self.test.upper()} CERT_TYPE={self.settings.cert} .evergreen/scripts/run-ocsp-test.sh
                        """
                    )
                )
        else:
            if self.test == "cache":
                yield (
                    shell_mongoc(
                        f"""
                        CERT_TYPE={self.settings.cert} .evergreen/scripts/run-ocsp-cache-test.sh
                        """
                    )
                )
            else:
                yield (
                    shell_mongoc(
                        f"""
                        TEST_COLUMN={self.test.upper()} CERT_TYPE={self.settings.cert} .evergreen/scripts/run-ocsp-test.sh
                        """
                    )
                )

    def to_dict(self):
        task = super(MatrixTask, self).to_dict()

        # OCSP tests should run with a batchtime of 14 days. Avoid running OCSP
        # tests in patch builds by default (only in commit builds).
        task["patchable"] = False

        return task

    # Testing in OCSP has a lot of exceptions.
    def do_is_valid_combination(self) -> bool:
        if self.settings.ssl == "darwinssl":
            # Secure Transport quietly ignores a must-staple certificate with no stapled response.
            prohibit(self.test == "malicious_server_test_2")

        # ECDSA certs can't be loaded (in the PEM format they're stored) on Windows/macOS. Skip them.
        if self.settings.ssl == "darwinssl" or self.settings.ssl == "winssl":
            prohibit(self.settings.cert == "ecdsa")

        # OCSP stapling is not supported on macOS or Windows.
        if self.settings.ssl == "darwinssl" or self.settings.ssl == "winssl":
            prohibit(self.test in ["test_1", "test_2", "cache"])

        if self.test == "soft_fail_test" or self.test == "malicious_server_test_2" or self.test == "cache":
            prohibit(self.settings.delegate == "delegate")
        return True


all_tasks = chain(all_tasks, OCSPTask.matrix())

all_tasks = list(all_tasks)



from shrub.v3.evg_command import attach_results

from config_generator.etc.function import Function
from config_generator.etc.utils import bash_exec


class UploadTestResults(Function):
    name = 'upload-test-results'
    commands = [
        # Ensure attach_results does not fail even if no tests results exist.
        bash_exec(
            script='''\
                mkdir -p mongoc
                touch mongoc/test-results.json
            '''
        ),
        attach_results(
            file_location='mongoc/test-results.json'
        ),
    ]

    @classmethod
    def call(cls, **kwargs):
        return cls.default_call(**kwargs)


def functions():
    return UploadTestResults.defn()



# Copyright 2009-present MongoDB, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import Iterable, Mapping
from evergreen_config_generator import ConfigObject

from . import ValueMapping


class Variant(ConfigObject):
    def __init__(
        self,
        name: str,
        display_name: str,
        run_on: list[str] | str,
        tasks: Iterable[str | ValueMapping],
        expansions: Mapping[str, str] | None = None,
        tags: Iterable[str] | None = None,
        patchable: bool | None = None,
        batchtime: int | None = None,
        display_tasks: Iterable[ValueMapping] = None,
    ):
        super(Variant, self).__init__()
        self._variant_name = name
        self.display_name = display_name
        self.run_on = run_on
        self.tasks = tasks
        self.expansions = expansions
        self.tags = tags
        self.patchable = patchable
        self.batchtime = batchtime
        self.display_tasks = display_tasks

    @property
    def name(self):
        return self._variant_name

    def to_dict(self):
        v = super(Variant, self).to_dict()
        for i in "display_name", "expansions", "run_on", "tasks", "patchable", "batchtime", "tags", "display_tasks":
            attr = getattr(self, i)

            # Allow `False`, but ignore empty lists and dicts.
            if isinstance(attr, None | list | dict) and not attr:
                continue

            v[i] = attr
        return v



from itertools import product

from shrub.v3.evg_command import expansions_update
from shrub.v3.evg_command import KeyValueParam
from shrub.v3.evg_task import EvgTask, EvgTaskDependency

from config_generator.etc.distros import find_small_distro
from config_generator.etc.distros import make_distro_str
from config_generator.etc.distros import to_cc

from config_generator.components.funcs.bootstrap_mongo_orchestration import BootstrapMongoOrchestration
from config_generator.components.funcs.fetch_build import FetchBuild
from config_generator.components.funcs.fetch_det import FetchDET
from config_generator.components.funcs.run_simple_http_server import RunSimpleHTTPServer
from config_generator.components.funcs.run_tests import RunTests


def generate_test_tasks(SSL, TAG, MATRIX):
    res = []

    TOPOLOGY_TO_STR = {
        'server': 'server',
        'replica': 'replica_set',
        'sharded': 'sharded_cluster',
    }

    for distro_name, compiler, arch, sasl, auths, topologies, server_vers in MATRIX:
        tags = [TAG, 'test', distro_name, compiler]
        test_distro = find_small_distro(distro_name)

        compile_vars = []
        compile_vars.append(KeyValueParam(key='CC', value=to_cc(compiler)))

        if arch:
            tags.append(arch)
            compile_vars.append(KeyValueParam(key='MARCH', value=arch))

        distro_str = make_distro_str(distro_name, compiler, arch)
        compile_task_name = f'sasl-{sasl}-{SSL}-{distro_str}-compile'

        for auth, topology, server_ver in product(auths, topologies, server_vers):
            test_tags = tags + [f'sasl-{sasl}', auth, topology, server_ver]
            # Do not add `nossl` tag to prevent being selected by legacy config variants.
            # Remove the `if` when CDRIVER-4571 is resolved.
            if SSL != 'nossl':
                test_tags += [SSL]

            test_task_name = f'sasl-{sasl}-{SSL}-{distro_str}-test-{server_ver}-{topology}-{auth}'

            test_commands = []
            test_commands.append(FetchBuild.call(build_name=compile_task_name))

            updates = compile_vars + [
                KeyValueParam(key='AUTH', value=auth),
                KeyValueParam(key='MONGODB_VERSION', value=server_ver),
                KeyValueParam(key='TOPOLOGY', value=TOPOLOGY_TO_STR[topology]),
                KeyValueParam(key='SSL', value=SSL),
            ]

            test_commands.append(expansions_update(updates=updates))
            test_commands.append(FetchDET.call())
            test_commands.append(BootstrapMongoOrchestration.call())
            test_commands.append(RunSimpleHTTPServer.call())
            test_commands.append(RunTests.call())

            res.append(
                EvgTask(
                    name=test_task_name,
                    run_on=test_distro.name,
                    tags=test_tags,
                    depends_on=[EvgTaskDependency(name=compile_task_name)],
                    commands=test_commands,
                )
            )

    return res



# Copyright 2009-present MongoDB, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from collections import OrderedDict as OD
import copy
from itertools import chain, product
import itertools
from typing import ClassVar, Iterable, Literal, Mapping, MutableMapping, Sequence, Union

from evergreen_config_generator import ConfigObject
from evergreen_config_generator.functions import func

from . import Value, MutableValueMapping, ValueSequence


DependencySpec = Union[str, Mapping[str, Value]]


class Task(ConfigObject):
    def __init__(
        self,
        task_name: str | None = None,
        commands: Iterable[Value] = (),
        tags: Iterable[str] = (),
        depends_on: Iterable[DependencySpec] = (),
        exec_timeout_secs: int | None = None,
    ):
        self._name = task_name
        self._tags = list(tags)
        self.options: MutableValueMapping = OD()
        self.commands: ValueSequence = list(commands)
        self.exec_timeout_secs = exec_timeout_secs
        self._depends_on = list(map(self._normal_dep, depends_on))

        if exec_timeout_secs is not None:
            self.options["exec_timeout_secs"] = exec_timeout_secs

    @property
    def dependencies(self) -> Sequence[Mapping[str, Value]]:
        main = list(self._depends_on)
        main.extend(map(self._normal_dep, self.additional_dependencies()))
        return tuple(main)

    def _normal_dep(self, spec: DependencySpec) -> Mapping[str, Value]:
        if isinstance(spec, str):
            return OD([("name", spec)])
        return spec

    @property
    def tags(self) -> Sequence[str]:
        return tuple(sorted(chain(self.additional_tags(), self._tags)))

    def pre_commands(self) -> Iterable[Value]:
        return ()

    def main_commands(self) -> Iterable[Value]:
        return ()

    def post_commands(self) -> Iterable[Value]:
        return ()

    def additional_dependencies(self) -> Iterable[DependencySpec]:
        return ()

    @property
    def name(self) -> str:
        assert self._name is not None, f'Task {self} did not set a name, and did not override the "name" property'
        return self._name

    def additional_tags(self) -> Iterable[str]:
        return ()

    def add_dependency(self, dependency: DependencySpec):
        if isinstance(dependency, str):
            dependency = OD([("name", dependency)])

        self._depends_on.append(dependency)

    def to_dict(self):
        task: MutableValueMapping = super().to_dict()  # type: ignore
        assert isinstance(task, MutableMapping)
        if self.tags:
            task["tags"] = list(self.tags)
        task.update(self.options)
        deps: Sequence[MutableValueMapping] = list(self.dependencies)  # type: ignore
        if deps:
            if len(deps) == 1:
                task["depends_on"] = OD(deps[0])
            else:
                task["depends_on"] = copy.deepcopy(deps)
        task["commands"] = list(
            itertools.chain(
                self.pre_commands(),
                self.main_commands(),
                self.commands,
                self.post_commands(),
            )
        )
        return task


NamedTask = Task


class FuncTask(NamedTask):
    def __init__(
        self,
        task_name: str,
        functions: Iterable[str],
        tags: Iterable[str] = (),
        depends_on: Iterable[DependencySpec] = (),
        exec_timeout_secs: int | None = None,
    ):
        commands = [func(func_name) for func_name in functions]
        super().__init__(task_name, commands, tags=tags, depends_on=depends_on, exec_timeout_secs=exec_timeout_secs)
        super(FuncTask, self).__init__(task_name, commands=commands)


class Prohibited(Exception):
    pass


def require(rule: bool) -> None:
    if not rule:
        raise Prohibited()


def prohibit(rule: bool) -> None:
    if rule:
        raise Prohibited()


def both_or_neither(rule0: bool, rule1: bool) -> None:
    if rule0:
        require(rule1)
    else:
        prohibit(rule1)


class SettingsAccess:
    def __init__(self, inst: "MatrixTask") -> None:
        self._task = inst

    def __getattr__(self, __setting: str) -> str | bool:
        return self._task.setting_value(__setting)


class MatrixTask(Task):
    axes: ClassVar[Mapping[str, Sequence[str | bool]]] = OD()

    def __init__(self, settings: Mapping[str, str | bool]):
        super().__init__()
        self._settings = {k: v for k, v in settings.items()}
        for axis, options in type(self).axes.items():
            if axis not in self._settings:
                self._settings[axis] = options[0]

    def display(self, axis_name: str) -> str:
        value = self.setting_value(axis_name)
        if value is False:
            # E.g., if self.auth is False, return 'noauth'.
            return f"no{axis_name}"
        elif value is True:
            return axis_name
        else:
            return value

    def on_off(self, key: str, val: str) -> Literal["on", "off"]:
        return "on" if self.setting_value(key) == val else "off"

    @property
    def name(self) -> str:
        return "-".join(self.name_parts())

    def name_parts(self) -> Iterable[str]:
        raise NotImplementedError

    @property
    def settings(self) -> SettingsAccess:
        return SettingsAccess(self)

    def setting_value(self, axis: str) -> str | bool:
        assert (
            axis in type(self).axes.keys()
        ), f'Attempted to inspect setting "{axis}", which is not defined for this task type'
        return self._settings[axis]

    def setting_eq(self, axis: str, val: str | bool) -> bool:
        current = self.setting_value(axis)
        options = type(self).axes[axis]
        assert (
            val in options
        ), f'Looking for value "{val}" on setting "{axis}", but that is not a supported option (Expects one of {options})'
        return current == val

    def is_valid_combination(self) -> bool:
        try:
            return self.do_is_valid_combination()
        except Prohibited:
            print(f"Ignoring invalid combination {self.name!r}")
            return False

    def do_is_valid_combination(self) -> bool:
        return True

    @classmethod
    def matrix(cls):
        for cell in product(*cls.axes.values()):
            axis_values = dict(zip(cls.axes, cell))
            task = cls(settings=axis_values)
            if task.allowed:
                yield task

    @property
    def allowed(self):
        try:
            return self.do_is_valid_combination()
        except Prohibited:
            return False



from config_generator.etc.function import Function
from config_generator.etc.utils import bash_exec


class StopLoadBalancer(Function):
    name = 'stop-load-balancer'
    commands = [
        bash_exec(
            script='''\
                # Only run if a load balancer was started.
                if [[ -z "${SINGLE_MONGOS_LB_URI}" ]]; then
                    echo "OK - no load balancer running"
                    exit
                fi
                if [[ -d drivers-evergreen-tools ]]; then
                    cd drivers-evergreen-tools && .evergreen/run-load-balancer.sh stop
                fi
            '''
        ),
    ]

    @classmethod
    def call(cls, **kwargs):
        return cls.default_call(**kwargs)


def functions():
    return StopLoadBalancer.defn()



#!/usr/bin/env python3

# A simple HTTP server used by /http tests.

import http
from http.server import BaseHTTPRequestHandler, HTTPServer


class Simple(BaseHTTPRequestHandler):
    def do_GET(self):
        self.send_response(http.HTTPStatus.OK)
        self.send_header('Content-type', 'text/html')
        self.end_headers()
        self.wfile.write(
            'Response to GET by simple HTTP server'.encode('utf-8'))

    def do_POST(self):
        self.send_response(http.HTTPStatus.OK)
        self.send_header('Content-type', 'text/html')
        self.end_headers()
        self.wfile.write(
            'Response to POST by simple HTTP server'.encode('utf-8'))


def main():
    HTTPServer(server_address=('', 18000),
               RequestHandlerClass=Simple).serve_forever()


if __name__ == '__main__':
    main()



from config_generator.etc.function import Function
from config_generator.etc.utils import bash_exec


class Backtrace(Function):
    name = 'backtrace'
    commands = [
        bash_exec(
            working_dir='mongoc',
            script='.evergreen/scripts/debug-core-evergreen.sh',
        ),
    ]

    @classmethod
    def call(cls, **kwargs):
        return cls.default_call(**kwargs)


def functions():
    return Backtrace.defn()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# Copyright 2009-present MongoDB, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

"""
A script that calculates the release version number (based on the current Git
branch and/or recent tags in history) to assign to a tarball generated from the
current Git commit.

This script needs to remain compatible with its target platforms, which currently
includes RHEL 6, which uses Python 2.6!
"""

# XXX NOTE XXX NOTE XXX NOTE XXX
# After modifying this script it is advisable to execute the self-tests in this directory:
# - calc_release_version_selftest.sh
# - calc_release_version_selftest.py
# XXX NOTE XXX NOTE XXX NOTE XXX

# pyright: reportTypeCommentUsage=false

import datetime
import errno
import re
import subprocess
import optparse  # No 'argparse' on Python 2.6
import sys


class Version:
    def __init__(self, s):
        pat = r'(\d+)\.(\d+)\.(\d+)(\-\S+)?'
        match = re.match(pat, s)
        assert match, "Unrecognized version string %s" % s
        self.major, self.minor, self.micro = (
            map(int, (match.group(1), match.group(2), match.group(3))))

        if match.group(4):
            self.prerelease = match.group(4)[1:]
        else:
            self.prerelease = ''

    def __lt__(self, other):
        if self.major != other.major:
            return self.major < other.major
        if self.minor != other.minor:
            return self.minor < other.minor
        if self.micro != other.micro:
            return self.micro < other.micro
        if self.prerelease != other.prerelease:
            if self.prerelease != '' and other.prerelease == '':
                # Consider a prerelease less than non-prerelease.
                return True
            # For simplicity, compare prerelease versions lexicographically.
            return self.prerelease < other.prerelease

        # Versions are equal.
        return False

    def __eq__(self, other):
        self_tuple = self.major, self.minor, self.micro, self.prerelease
        other_tuple = other.major, other.minor, other.micro, other.prerelease
        return self_tuple == other_tuple


def parse_version(ver):
    return Version(ver)


parser = optparse.OptionParser(description=__doc__)
parser.add_option("--debug", "-d", action="store_true", help="Enable debug output")
parser.add_option("--previous", "-p", action="store_true", help="Calculate the previous version instead of the current")
parser.add_option("--next-minor", action="store_true", help="Calculate the next minor version instead of the current")
args, pos = parser.parse_args()
assert not pos, "No positional arguments are expected"


_DEBUG = args.debug  # type: bool


def debug(msg):  # type: (str) -> None
    if _DEBUG:
        sys.stderr.write(msg)
        sys.stderr.write("\n")
        sys.stderr.flush()


debug("Debugging output enabled.")

# This option indicates we are to determine the previous release version
PREVIOUS = args.previous  # type: bool
# This options indicates to output the next minor release version
NEXT_MINOR = args.next_minor  # type: bool

# fmt: off

PREVIOUS_TAG_RE = re.compile('(?P<ver>(?P<vermaj>[0-9]+)\\.(?P<vermin>[0-9]+)'
                             '\\.(?P<verpatch>[0-9]+)(?:-(?P<verpre>.*))?)')
RELEASE_TAG_RE = re.compile('(?P<ver>(?P<vermaj>[0-9]+)\\.(?P<vermin>[0-9]+)'
                            '\\.(?P<verpatch>[0-9]+)(?:-(?P<verpre>.*))?)')
RELEASE_BRANCH_RE = re.compile('(?:(?:refs/remotes/)?origin/)?(?P<brname>r'
                               '(?P<vermaj>[0-9]+)\\.(?P<vermin>[0-9]+))')


def check_output(args):  # type: (list[str]) -> str
    """
    Delegates to subprocess.check_output() if it is available, otherwise
    provides a reasonable facsimile.
    """
    debug('Run command: {0}'.format(args))
    try:
        proc = subprocess.Popen(args, stdout=subprocess.PIPE)
    except OSError as e:
        suppl = ''
        if e.errno == errno.ENOENT:
            suppl = 'Does the executable “{0}” not exist?'.format(args[0])
        raise RuntimeError("Failed to execute subprocess {0}: {1} [{2}]".format(args, e, suppl))
    out = proc.communicate()[0]
    ret = proc.poll()
    if ret:
        raise subprocess.CalledProcessError(ret, args[0])

    # git isn't guaranteed to always return UTF-8, but for our purposes
    # this should be fine as tags and hashes should be ASCII only.
    out = out.decode('utf-8')

    return out


def check_head_tag():  # type: () -> str | None
    """
    Checks the current HEAD to see if it has been tagged with a tag that matches
    the pattern for a release tag.  Returns release version calculated from the
    tag, or None if there is no matching tag associated with HEAD.
    """

    found_tag = False
    version_str = '0.0.0'
    version_parsed = parse_version(version_str)

    # have git tell us if any tags that look like release tags point at HEAD;
    # based on our policy, a commit should never have more than one release tag
    tags = check_output(['git', 'tag', '--points-at', 'HEAD', '--list', '1.*']).split()
    tag = ''
    if len(tags) == 1:
        tag = tags[0]
    elif len(tags) > 1:
        raise Exception('Expected 1 or 0 tags on HEAD, got: {}'.format(tags))

    release_tag_match = RELEASE_TAG_RE.match(tag)
    if release_tag_match:
        new_version_str = release_tag_match.group('ver')
        new_version_parsed = parse_version(new_version_str)
        if new_version_parsed > version_parsed: # type: ignore
            debug('HEAD release tag: ' + new_version_str)
            version_str = new_version_str
            version_parsed = new_version_parsed
            found_tag = True

    if found_tag:
        debug('Calculated version: ' + version_str)
        return version_str

    return None

def get_next_minor(prerelease_marker):  # type: (str) -> str
    """
    get_next_minor does the following:
        - Inspect the branches that fit the convention for a release branch.
        - Choose the latest and increment the minor version.
        - Append .0 to form the new version (e.g., r1.21 becomes 1.22.0)
        - Append a pre-release marker. (e.g. 1.22.0 becomes 1.22.0-20220201+gitf6e6a7025d)
    """

    version_str = '0.0.0'
    version_parsed = parse_version(version_str)

    version_new = {}
    # Use refs (not branches) to get local branches plus remote branches
    refs = check_output(['git', 'show-ref']).splitlines()
    for ref in refs:
        release_branch_match = RELEASE_BRANCH_RE.match(ref.split()[1])
        if release_branch_match:
            # Construct a candidate version from this branch name
            version_new['major'] = int(release_branch_match.group('vermaj'))
            version_new['minor'] = int(release_branch_match.group('vermin')) + 1
            version_new['patch'] = 0
            version_new['prerelease'] = prerelease_marker
            new_version_str = str(version_new['major']) + '.' + \
                              str(version_new['minor']) + '.' + \
                              str(version_new['patch']) + '-' + \
                              version_new['prerelease']
            new_version_parsed = parse_version(new_version_str)
            if new_version_parsed > version_parsed: # type: ignore
                version_str = new_version_str
                version_parsed = new_version_parsed
                debug('Found new best version "' + version_str \
                            + '" based on branch "' \
                            + release_branch_match.group('brname') + '"')
    return version_str

def get_branch_tags(active_branch_name):  # type: (str) -> list[str]
    """
    Returns a list of tags corresponding to the current branch, which must not
    be master.  If the specified branch is a release branch then return all tags
    based on the major/minor X.Y release version.  If the specified branch is
    neither master nor a release branch, then walk backwards in history until
    the first tag matching the glob '1.*' and return that tag.
    """

    if active_branch_name == 'master':
        raise Exception('this method is not meant to be called while on "master"')

    release_branch_match = RELEASE_BRANCH_RE.match(active_branch_name)
    if release_branch_match:
        # This is a release branch, so look for tags only on this branch
        tag_glob = release_branch_match.group('vermaj') + '.' \
                + release_branch_match.group('vermin') + '.*'
        return check_output(['git', 'tag', '--list', tag_glob]).splitlines()

    # Not a release branch, so look for the most recent tag in history
    commits = check_output(['git', 'log', '--pretty=format:%H', '--no-merges'])
    tags_by_obj = get_object_tags()
    for commit in commits.splitlines():
        got = tags_by_obj.get(commit)
        if got:
            return got
    # No tags
    return []


def iter_tag_lines():
    """
    Generate a list of pairs of strings, where the first is a commit hash, and
    the second is a tag that is associated with that commit. Duplicate commits
    are possible.
    """
    output = check_output(['git', 'tag', '--list', '1.*', '--format=%(*objectname)|%(tag)'])
    lines = output.splitlines()
    for l in lines:
        obj, tag = l.split('|', 1)
        if tag:
            yield obj, tag


def get_object_tags():  # type: () -> dict[str, list[str]]
    """
    Obtain a mapping between commit hashes and a list of tags that point to
    that commit. Untagged commits will not be included in the resulting map.
    """
    ret = {}  # type: dict[str, list[str]]
    for obj, tag in iter_tag_lines():
        ret.setdefault(obj, []).append(tag)
    return ret


def process_and_sort_tags(tags):  # type: (list[str]) -> list[str]
    """
    Given a string (as returned from get_branch_tags), return a sorted list of
    zero or more tags (sorted based on the Version comparison) which meet
    the following criteria:
        - a final release tag (i.e., 1.x.y without any pre-release suffix)
        - a pre-release tag which is not superseded by a release tag (i.e.,
          1.x.y-preX iff 1.x.y does not already exist)
    """

    processed_and_sorted_tags = []  # type: list[str]
    if not tags or len(tags) == 0:
        return processed_and_sorted_tags

    # find all the final release tags
    for tag in tags:
        release_tag_match = RELEASE_TAG_RE.match(tag)
        if release_tag_match and not release_tag_match.group('verpre'):
            processed_and_sorted_tags.append(tag)
    # collect together final release tags and pre-release tags for
    # versions that have not yet had a final release
    for tag in tags:
        tag_parts = tag.split('-')
        if len(tag_parts) >= 2 and tag_parts[0] not in processed_and_sorted_tags:
            processed_and_sorted_tags.append(tag)
    processed_and_sorted_tags.sort(key=Version)  # type: ignore

    return processed_and_sorted_tags

def main():
    """
    The algorithm is roughly:

        - Is the --next-minor flag passed? If "yes", then return the next minor
           release with a pre-release marker.
        - Is the current HEAD associated with a tag that looks like a release
           version?
        - If "yes" then use that as the version
        - If "no" then is the current branch master?
        - If "yes" the current branch is master, then return the next minor
           release with a pre-release marker.
        - If "no" the current branch is not master, then determine the most
           recent tag in history; strip any pre-release marker, increment the
           patch version, and append a new pre-release marker
    """

    version_str = '0.0.0'
    version_parsed = parse_version(version_str)
    head_commit_short = check_output(['git', 'rev-parse', '--revs-only',
                                      '--short=10', 'HEAD^{commit}']).strip()
    prerelease_marker = datetime.date.today().strftime('%Y%m%d') \
            + '+git' + head_commit_short

    if NEXT_MINOR:
        debug('Calculating next minor release')
        return get_next_minor(prerelease_marker)

    head_tag_ver = check_head_tag()
    if head_tag_ver:
        return head_tag_ver

    active_branch_name = check_output(['git', 'rev-parse',
                                       '--abbrev-ref', 'HEAD']).strip()
    debug('Calculating release version for branch: ' + active_branch_name)
    if active_branch_name == 'master':
        return get_next_minor(prerelease_marker)

    branch_tags = get_branch_tags(active_branch_name)
    tags = process_and_sort_tags(branch_tags)

    tag = tags[-1] if len(tags) > 0 else ''
    # at this point the RE match is redundant, but convenient for accessing
    # the components of the version string
    release_tag_match = RELEASE_TAG_RE.match(tag)
    if release_tag_match:
        version_new = {}
        version_new['major'] = int(release_tag_match.group('vermaj'))
        version_new['minor'] = int(release_tag_match.group('vermin'))
        version_new['patch'] = int(release_tag_match.group('verpatch')) + 1
        version_new['prerelease'] = prerelease_marker
        new_version_str = str(version_new['major']) + '.' + \
                          str(version_new['minor']) + '.' + \
                          str(version_new['patch']) + '-' + \
                          version_new['prerelease']
        new_version_parsed = parse_version(new_version_str)
        if new_version_parsed > version_parsed: # type: ignore
            version_str = new_version_str
            version_parsed = new_version_parsed
            debug('Found new best version "' + version_str \
                        + '" from tag "' + release_tag_match.group('ver') + '"')

    return version_str

def previous(rel_ver):  # type: (str) -> str
    """
    Given a release version, find the previous version based on the latest Git
    tag that is strictly a lower version than the given release version.
    """
    debug('Calculating previous release version (option -p was specified).')
    version_str = '0.0.0'
    version_parsed = parse_version(version_str)
    rel_ver_str = rel_ver
    rel_ver_parsed = parse_version(rel_ver_str)
    tags = check_output(['git', 'tag', '--list', '1.*']).splitlines()
    processed_and_sorted_tags = process_and_sort_tags(tags)
    for tag in processed_and_sorted_tags:
        previous_tag_match = PREVIOUS_TAG_RE.match(tag)
        if previous_tag_match:
            version_new = {}
            version_new['major'] = int(previous_tag_match.group('vermaj'))
            version_new['minor'] = int(previous_tag_match.group('vermin'))
            version_new['patch'] = int(previous_tag_match.group('verpatch'))
            version_new['prerelease'] = previous_tag_match.group('verpre')
            new_version_str = str(version_new['major']) + '.' + \
                              str(version_new['minor']) + '.' + \
                              str(version_new['patch'])
            if version_new['prerelease'] is not None:
                new_version_str += '-' + version_new['prerelease']
            new_version_parsed = parse_version(new_version_str)
            if new_version_parsed < rel_ver_parsed and new_version_parsed > version_parsed: # type: ignore
                version_str = new_version_str
                version_parsed = new_version_parsed
                debug('Found new best version "' + version_str \
                            + '" from tag "' + tag + '"')

    return version_str

if __name__ == "__main__":
    RELEASE_VER = previous(main()) if PREVIOUS else main()

    debug('Final calculated release version:')
    print(RELEASE_VER)



from config_generator.etc.function import Function
from shrub.v3.evg_command import archive_targz_pack
from shrub.v3.evg_command import s3_put


class UploadBuild(Function):
    name = 'upload-build'
    commands = [
        archive_targz_pack(
            target='${build_id}.tar.gz',
            source_dir='mongoc',
            include=['./**'],
        ),
        s3_put(
            aws_key='${aws_key}',
            aws_secret='${aws_secret}',
            remote_file='mongo-c-driver/${build_variant}/${revision}/${task_name}/${build_id}.tar.gz',
            bucket='mciuploads',
            permissions='public-read',
            local_file='${build_id}.tar.gz',
            content_type='${content_type|application/x-gzip}',
        ),
    ]

    @classmethod
    def call(cls, **kwargs):
        return cls.default_call(**kwargs)


def functions():
    return UploadBuild.defn()



from shrub.v3.evg_project import EvgProject

from config_generator.etc import utils


def generate():
    functions = {}

    for component in utils.all_components():
        if hasattr(component, 'functions'):
            component_name = utils.component_name(component)
            print(f' - {component_name}')
            functions.update(component.functions())

    functions = dict(sorted(functions.items()))
    yml = utils.to_yaml(EvgProject(functions=functions))
    utils.write_to_file(yml, 'functions.yml')



#!/usr/bin/env python
#
# Copyright 2009-present MongoDB, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from collections import OrderedDict as OD
from typing import MutableSequence

from config_generator.components.funcs.find_cmake_latest import FindCMakeLatest

from evergreen_config_generator.functions import shell_exec, func
from evergreen_config_generator.tasks import NamedTask
from evergreen_config_generator.variants import Variant
from evergreen_config_generator.taskgroups import TaskGroup


def _create_tasks():
    # passtask is expected to run on a remote Azure VM and succeed at obtaining credentials.
    passtask = NamedTask(task_name="testazurekms-task")
    passtask.commands = [
        func("fetch-source"),
        func("find-cmake-latest"),
        shell_exec(
            r"""
            echo "Building test-azurekms ... begin"
            pushd mongoc
            .evergreen/scripts/compile-test-azurekms.sh
            popd
            echo "Building test-azurekms ... end"

            echo "Copying files ... begin"
            export AZUREKMS_RESOURCEGROUP=${testazurekms_resourcegroup}
            export AZUREKMS_VMNAME=${AZUREKMS_VMNAME}
            export AZUREKMS_PRIVATEKEYPATH=/tmp/testazurekms_privatekey
            DRIVERS_TOOLS=$(pwd)/drivers-evergreen-tools
            mkdir testazurekms
            cp ./mongoc/src/libmongoc/test-azurekms ./mongoc/install/lib/libmongocrypt.* testazurekms
            tar czf testazurekms.tgz testazurekms/*
            AZUREKMS_SRC="testazurekms.tgz" \
            AZUREKMS_DST="./" \
                $DRIVERS_TOOLS/.evergreen/csfle/azurekms/copy-file.sh
            echo "Copying files ... end"

            echo "Untarring file ... begin"
            AZUREKMS_CMD="tar xf testazurekms.tgz" \
                $DRIVERS_TOOLS/.evergreen/csfle/azurekms/run-command.sh
            echo "Untarring file ... end"
            """,
            test=False,
            add_expansions_to_env=True,
        ),
        shell_exec(
            r"""
            export AZUREKMS_RESOURCEGROUP=${testazurekms_resourcegroup}
            export AZUREKMS_VMNAME=${AZUREKMS_VMNAME}
            export AZUREKMS_PRIVATEKEYPATH=/tmp/testazurekms_privatekey
            DRIVERS_TOOLS=$(pwd)/drivers-evergreen-tools
            AZUREKMS_CMD="LD_LIBRARY_PATH=./testazurekms MONGODB_URI='mongodb://localhost:27017' KEY_NAME='${testazurekms_keyname}' KEY_VAULT_ENDPOINT='${testazurekms_keyvaultendpoint}' ./testazurekms/test-azurekms" \
                $DRIVERS_TOOLS/.evergreen/csfle/azurekms/run-command.sh
            """
        ),
    ]

    failtask = NamedTask(task_name="testazurekms-fail-task")
    failtask.commands = [
        func("fetch-source"),
        func("find-cmake-latest"),
        shell_exec(
            r"""
            pushd mongoc
            .evergreen/scripts/compile-test-azurekms.sh
            popd
            """,
            test=False,
            add_expansions_to_env=True,
        ),
        shell_exec(
            r"""
            LD_LIBRARY_PATH=$PWD/install \
            MONGODB_URI='mongodb://localhost:27017' \
            KEY_NAME='${testazurekms_keyname}' \
            KEY_VAULT_ENDPOINT='${testazurekms_keyvaultendpoint}' \
            EXPECT_ERROR='Error from Azure IMDS server' \
                ./mongoc/src/libmongoc/test-azurekms
            """
        ),
    ]
    return [passtask, failtask]


def _create_variant():
    return Variant(
        name="testazurekms-variant",
        display_name="Azure KMS",
        # Azure Virtual Machine created is Debian 10.
        run_on="debian10-small",
        tasks=["testazurekms_task_group", "testazurekms-fail-task"],
        batchtime=20160,
    )  # Use a batchtime of 14 days as suggested by the CSFLE test README


def _create_task_group():
    task_group = TaskGroup(name="testazurekms_task_group")
    task_group.setup_group_can_fail_task = True
    task_group.setup_group_timeout_secs = 1800  # 30 minutes
    task_group.setup_group = [
        func("fetch-det"),
        shell_exec(
            r"""
            DRIVERS_TOOLS=$(pwd)/drivers-evergreen-tools
            echo '${testazurekms_publickey}' > /tmp/testazurekms_publickey
            echo '${testazurekms_privatekey}' > /tmp/testazurekms_privatekey
            # Set 600 permissions on private key file. Otherwise ssh / scp may error with permissions "are too open".
            chmod 600 /tmp/testazurekms_privatekey
            export AZUREKMS_CLIENTID=${testazurekms_clientid}
            export AZUREKMS_TENANTID=${testazurekms_tenantid}
            export AZUREKMS_SECRET=${testazurekms_secret}
            export AZUREKMS_DRIVERS_TOOLS=$DRIVERS_TOOLS
            export AZUREKMS_RESOURCEGROUP=${testazurekms_resourcegroup}
            export AZUREKMS_PUBLICKEYPATH=/tmp/testazurekms_publickey
            export AZUREKMS_PRIVATEKEYPATH=/tmp/testazurekms_privatekey
            export AZUREKMS_SCOPE=${testazurekms_scope}
            export AZUREKMS_VMNAME_PREFIX=CDRIVER
            $DRIVERS_TOOLS/.evergreen/csfle/azurekms/create-and-setup-vm.sh
            """,
            test=False,
        ),
        # Load the AZUREKMS_VMNAME expansion.
        OD(
            [
                ("command", "expansions.update"),
                (
                    "params",
                    OD(
                        [
                            ("file", "testazurekms-expansions.yml"),
                        ]
                    ),
                ),
            ]
        ),
    ]

    task_group.teardown_group = [
        # Load expansions again. The setup task may have failed before running `expansions.update`.
        OD(
            [
                ("command", "expansions.update"),
                (
                    "params",
                    OD(
                        [
                            ("file", "testazurekms-expansions.yml"),
                        ]
                    ),
                ),
            ]
        ),
        shell_exec(
            r"""
                DRIVERS_TOOLS=$(pwd)/drivers-evergreen-tools
                export AZUREKMS_VMNAME=${AZUREKMS_VMNAME}
                export AZUREKMS_RESOURCEGROUP=${testazurekms_resourcegroup}
                $DRIVERS_TOOLS/.evergreen/csfle/azurekms/delete-vm.sh
            """,
            test=False,
        ),
    ]
    task_group.tasks = ["testazurekms-task"]
    return task_group


def testazurekms_generate(
    all_tasks: MutableSequence[NamedTask],
    all_variants: MutableSequence[Variant],
    all_task_groups: MutableSequence[TaskGroup],
):
    all_tasks.extend(_create_tasks())
    all_variants.append(_create_variant())
    all_task_groups.append(_create_task_group())



#!/usr/bin/env python3

# Copyright 2009-present MongoDB, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Check that there are no missing .c or .h files in the release tarball.
"""
import os
import re
import sys

if len(sys.argv) != 3:
    print("Usage: python check_files.py"
          " <src directory from repo>"
          " <src directory produced by tarball>")
    sys.exit(1)

REPO_SRC_DIR = sys.argv[1]
TAR_SRC_DIR = sys.argv[2]

pattern = re.compile(r".*\.(c|h)$")

repo_src_files = set()
tar_src_files = set()

# Files that are generated by cmake are expected not to be in the repository.
cmake_produced = [
    "libbson/src/bson/bson-version.h",
    "libmongoc/src/mongoc/mongoc-version.h",
    "zlib-1.2.13/zconf.h",
]

for root, dirs, files in os.walk(REPO_SRC_DIR):
    for file in files:
        relpath = os.path.relpath(os.path.join(root, file), REPO_SRC_DIR)

        if pattern.match(relpath):
            repo_src_files.add(relpath)

for root, dirs, files in os.walk(TAR_SRC_DIR):
    for file in files:
        relpath = os.path.relpath(os.path.join(root, file), TAR_SRC_DIR)
        if relpath in cmake_produced:
            continue
        if pattern.match(relpath):
            tar_src_files.add(relpath)

if tar_src_files != repo_src_files:
    print("Files produced from tarball differ from repository")
    print("Tarball produced the following files NOT in repository")
    for file in sorted(tar_src_files.difference(repo_src_files)):
        print(" " + file)
    print("Repository has the following files NOT in tarball")
    for file in sorted(repo_src_files.difference(tar_src_files)):
        print(" " + file)
    sys.exit(1)



import itertools
from importlib import import_module
from inspect import isclass
from pathlib import Path
from textwrap import dedent
from typing import (Any, Iterable, Literal, Mapping, Type, TypeVar,
                    Union, cast)

import yaml
from shrub.v3.evg_command import EvgCommandType, subprocess_exec
from shrub.v3.evg_project import EvgProject
from shrub.v3.shrub_service import ConfigDumper
from shrub.v3.evg_task import EvgTaskRef
from typing_extensions import get_args, get_origin, get_type_hints

T = TypeVar('T')


# Equivalent to EvgTaskRef but defines additional properties.
class TaskRef(EvgTaskRef):
    """
    An evergreen task reference model that also includes additional properties.

    (The shrub.py model is missing some properties)
    """

    batchtime: int | None = None


# Automatically formats the provided script and invokes it in Bash.
def bash_exec(
    script,
    *,
    include_expansions_in_env: Iterable[str] | None = None,
    working_dir: str | None = None,
    command_type: EvgCommandType | None = None,
    retry_on_failure: bool | None = None,
    env: Mapping[str, str] | None = None,
    **kwargs,
):
    ret = subprocess_exec(
        binary="bash",
        args=["-c", dedent(script)],
        include_expansions_in_env=list(include_expansions_in_env) if include_expansions_in_env else None,
        working_dir=working_dir,
        command_type=command_type,
        env=dict(env) if env else None,
        **kwargs,
    )

    if retry_on_failure is not None:
        ret.params |= {"retry_on_failure": retry_on_failure}

    return ret


def all_components():
    res = []

    # .evergreen/config_generator/etc/utils.py -> .evergreen/config_generator/components
    components_dir = Path(__file__).parent.parent / 'components'

    all_paths = components_dir.glob('**/*.py')

    for path in sorted(all_paths):
        component_path = path.relative_to(components_dir)
        component_str = str(component_path.with_suffix(''))  # Drop '.py'.
        component_str = component_str.replace('/', '.')  # 'a/b' -> 'a.b'
        module_name = f'config_generator.components.{component_str}'
        res.append(import_module(module_name))

    return res


# Helper function to print component name for diagnostic purposes.
def component_name(component):
    component_prefix = 'config_generator.components.'
    res = component.__name__[len(component_prefix):]
    return res


def write_to_file(yml, filename):
    # .evergreen/config_generator/etc/utils.py -> .evergreen
    evergreen_dir = Path(__file__).parent.parent.parent
    filename = evergreen_dir / 'generated_configs' / filename

    with open(filename.resolve(), 'w', encoding='utf-8') as file:
        file.write(yml)


class Dumper(ConfigDumper):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        # List all tags on a single line.
        self.FLOW_TAG_COUNT = float('inf')

    # Make an effort to order fields in a readable manner.
    # Ordering applies to *all* mappings regardless of the parent node.
    def represent_mapping(self, tag, mapping, flow_style=False):
        if len(mapping) == 2 and 'key' in mapping and 'value' in mapping:
            flow_style = True

        before = [
            'name',
            'display_name',
            'command',
            'type',
            'run_on',
            'tags',
            'depends_on',
            'binary',
            'working_dir',
        ]

        after = [
            'commands',
            'args',
        ]

        ordered = {
            field: mapping.pop(field) for field in before if field in mapping
        }

        suffix = {
            field: mapping.pop(field) for field in after if field in mapping
        }

        ordered.update(sorted(mapping.items()))
        ordered.update(suffix)

        return self.represent_special_mapping(tag, ordered.items(), flow_style)


def to_yaml(project: EvgProject) -> str:
    return yaml.dump(
        project.dict(exclude_none=True, exclude_unset=True, by_alias=True),
        Dumper=Dumper,
        default_flow_style=False,
        width=float('inf'),
    )


def all_possible(typ: Type[T]) -> Iterable[T]:
    """
    Given a finite type, enumerate all possible values of that type.
    The following are "finite" types:

    - Literal[...] types
    - Union[...] of finite types
    - NamedTuple where each field is a finite type
    - None
    """
    origin = get_origin(typ)
    if typ is type(None):
        yield cast(T, None)
    elif origin is Literal:
        # It is a literal type, so enumerate its literal operands
        yield from get_args(typ)
        return
    elif origin == Union:
        args = get_args(typ)
        yield from itertools.chain.from_iterable(map(all_possible, args))
    elif isclass(typ) and issubclass(typ, tuple):
        # Iter each NamedTuple field:
        fields: Iterable[tuple[str, type]] = get_type_hints(typ).items()
        # Generate lists of pairs of field names and their possible values
        all_pairs: Iterable[Iterable[tuple[str, str]]] = (
            # Generate a (key, opt) pair for each option for 'key'
            [(key, opt) for opt in all_possible(typ)]
            # Over each field and type thereof:
            for key, typ in fields
        )
        # Now generate the cross product of all alternative for all fields:
        matrix: Iterable[dict[str, Any]] = map(dict, itertools.product(*all_pairs))
        for items in matrix:
            # Reconstruct as a NamedTuple:
            yield typ(**items)  # type: ignore
    else:
        raise TypeError(
            f'Do not know how to do "all_possible" of type {typ!r} ({origin=})'
        )



# -*- coding: utf-8 -*-
import os
import os.path
import sys

# Ensure we can import "mongoc" extension module.
this_path = os.path.dirname(__file__)
sys.path.append(os.path.normpath(os.path.join(this_path, "../../../build/sphinx")))

from mongoc_common import *

extensions = [
    "mongoc",
]

# General information about the project.
project = "libbson"
copyright = "2009-present, MongoDB, Inc."
author = "MongoDB, Inc"

version_path = os.path.join(os.path.dirname(__file__), "../../..", "VERSION_CURRENT")
version = open(version_path).read().strip()

language = "en"
exclude_patterns = ["_build", "Thumbs.db", ".DS_Store"]
master_doc = "index"

# -- Options for HTML output ----------------------------------------------

html_theme = "furo"
html_title = html_shorttitle = "libbson %s" % version
# html_favicon = None

html_sidebars = {}

html_use_index = False

rst_prolog = rf"""

.. _mongodb_docs_cdriver: https://www.mongodb.com/docs/languages/c/c-driver/current/

.. _errors: https://www.mongodb.com/docs/languages/c/c-driver/current/libbson/tutorials/errors/

"""


def add_canonical_link(app, pagename, templatename, context, doctree):
    link = f'<link rel="canonical" href="https://www.mongoc.org/libbson/current/{pagename}"/>'

    context["metatags"] = context.get("metatags", "") + link


def setup(app):
    mongoc_common_setup(app)
    app.connect("html-page-context", add_canonical_link)



# Copyright 2009-present MongoDB, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from collections import OrderedDict as OD

from evergreen_config_generator.variants import Variant


mobile_flags = (
    " -DCMAKE_FIND_ROOT_PATH_MODE_LIBRARY=ONLY"
    " -DCMAKE_FIND_ROOT_PATH_MODE_PACKAGE=ONLY"
    " -DCMAKE_FIND_ROOT_PATH_MODE_PROGRAM=NEVER"
    " -DCMAKE_FIND_ROOT_PATH_MODE_INCLUDE=ONLY"
)


def days(n: int) -> int:
    "Calculate the number of minutes in the given number of days"
    return n * 24 * 60


all_variants = [
    Variant(
        "abi-compliance-check",
        "ABI Compliance Check",
        ["ubuntu2004-small", "ubuntu2004-medium", "ubuntu2004-large"],
        ["abi-compliance-check"],
    ),
    Variant(
        "smoke",
        "Smoke Tests",
        "ubuntu2204-small",
        [
            "make-docs",
            "kms-divergence-check",
            "release-compile",
            "debug-compile-no-counters",
            "compile-tracing",
            "link-with-cmake",
            "link-with-cmake-deprecated",
            "link-with-cmake-ssl",
            "link-with-cmake-ssl-deprecated",
            "link-with-cmake-snappy",
            "link-with-cmake-snappy-deprecated",
            OD([("name", "link-with-cmake-mac"), ("distros", ["macos-14-arm64"])]),
            OD([("name", "link-with-cmake-mac-deprecated"), ("distros", ["macos-14-arm64"])]),
            OD([("name", "link-with-cmake-windows"), ("distros", ["windows-vsCurrent-large"])]),
            OD([("name", "link-with-cmake-windows-ssl"), ("distros", ["windows-vsCurrent-large"])]),
            OD([("name", "link-with-cmake-windows-snappy"), ("distros", ["windows-vsCurrent-large"])]),
            OD([("name", "link-with-cmake-mingw"), ("distros", ["windows-vsCurrent-large"])]),
            OD([("name", "link-with-pkg-config"), ("distros", ["ubuntu2004-test"])]),
            OD([("name", "link-with-pkg-config-mac"), ("distros", ["macos-14-arm64"])]),
            "link-with-pkg-config-ssl",
            "link-with-bson",
            OD([("name", "link-with-bson-windows"), ("distros", ["windows-vsCurrent-large"])]),
            OD([("name", "link-with-bson-mac"), ("distros", ["macos-14-arm64"])]),
            OD([("name", "link-with-bson-mingw"), ("distros", ["windows-vsCurrent-large"])]),
            "check-headers",
            "install-uninstall-check",
            OD([("name", "install-uninstall-check-mingw"), ("distros", ["windows-vsCurrent-large"])]),
            OD([("name", "install-uninstall-check-msvc"), ("distros", ["windows-vsCurrent-large"])]),
            "debug-compile-with-warnings",
            OD([("name", "build-and-test-with-toolchain"), ("distros", ["debian10-small"])]),
            "install-libmongoc-after-libbson",
        ],
        {
            # Disable the MongoDB legacy shell download, which is not available in 5.0 for u22
            "SKIP_LEGACY_SHELL": "1"
        },
        tags=["pr-merge-gate"],
    ),
    Variant(
        "openssl",
        "OpenSSL / LibreSSL",
        "archlinux-build",
        [
            "build-and-run-authentication-tests-openssl-1.0.1",
            "build-and-run-authentication-tests-openssl-1.0.2",
            "build-and-run-authentication-tests-openssl-1.1.0",
            "build-and-run-authentication-tests-openssl-1.0.1-fips",
            "build-and-run-authentication-tests-libressl-2.5",
            "build-and-run-authentication-tests-libressl-3.0-auto",
            "build-and-run-authentication-tests-libressl-3.0",
        ],
        {},
    ),
    Variant(
        "clang37",
        "clang 3.7 (Archlinux)",
        "archlinux-test",
        [
            "release-compile",
            "debug-compile-sasl-openssl",
            "debug-compile-nosasl-openssl",
            ".authentication-tests .openssl",
        ],
        {"CC": "clang"},
    ),
    Variant(
        "clang100-i686",
        "clang 10.0 (i686) (Ubuntu 20.04)",
        "ubuntu2004-test",
        [
            "release-compile",
            "debug-compile-nosasl-nossl",
            "debug-compile-no-align",
            ".debug-compile !.sspi .nossl .nosasl",
            ".latest .nossl .nosasl",
        ],
        {"CC": "clang", "MARCH": "i686"},
    ),
    Variant(
        "gcc82rhel",
        "GCC 8.2 (RHEL 8.0)",
        "rhel80-test",
        [
            ".hardened",
            ".compression !.snappy !.zstd",
            "release-compile",
            "debug-compile-nosasl-nossl",
            "debug-compile-nosasl-openssl",
            "debug-compile-sasl-openssl",
            ".authentication-tests .openssl",
            ".latest .nossl",
        ],
        {"CC": "gcc"},
    ),
    Variant(
        "gcc83",
        "GCC 8.3 (Debian 10.0)",
        "debian10-test",
        ["release-compile", "debug-compile-nosasl-nossl", ".latest .nossl"],
        {"CC": "gcc"},
    ),
    Variant(
        "gcc102",
        "GCC 10.2 (Debian 11.0)",
        "debian11-large",
        ["release-compile", "debug-compile-nosasl-nossl", ".latest .nossl"],
        {"CC": "gcc"},
    ),
    Variant(
        "gcc94-i686",
        "GCC 9.4 (i686) (Ubuntu 20.04)",
        "ubuntu2004-test",
        ["release-compile", "debug-compile-nosasl-nossl", "debug-compile-no-align", ".latest .nossl .nosasl"],
        {"CC": "gcc", "MARCH": "i686"},
    ),
    Variant(
        "gcc94",
        "GCC 9.4 (Ubuntu 20.04)",
        "ubuntu2004-test",
        [
            ".compression !.zstd",
            "debug-compile-nosrv",
            "release-compile",
            "debug-compile-nosasl-nossl",
            "debug-compile-no-align",
            "debug-compile-sasl-openssl",
            "debug-compile-nosasl-openssl",
            ".authentication-tests .openssl",
            ".authentication-tests .asan",
            ".test-coverage",
            ".latest .nossl",
            "retry-true-latest-server",
            "test-dns-openssl",
            "test-dns-auth-openssl",
            "test-dns-loadbalanced-openssl",
        ],
        {"CC": "gcc"},
    ),
    Variant(
        "darwin",
        "*Darwin, macOS (Apple LLVM)",
        "macos-14-arm64",
        [
            ".compression !.snappy",
            "release-compile",
            "debug-compile-nosasl-nossl",
            "debug-compile-no-align",
            "debug-compile-nosrv",
            "debug-compile-sasl-darwinssl",
            "debug-compile-nosasl-nossl",
            ".authentication-tests .darwinssl",
            ".latest .nossl",
            "test-dns-darwinssl",
            "test-dns-auth-darwinssl",
            "debug-compile-lto",
            "debug-compile-lto-thin",
            "debug-compile-aws",
            "test-aws-openssl-regular-latest",
        ],
        {"CC": "clang"},
    ),
    Variant(
        "darwin-intel",
        "*Darwin, Intel macOS (Apple LLVM)",
        "macos-14",
        [
            "debug-compile-aws",
            "debug-compile-rdtscp",
            "test-aws-openssl-regular-4.4",
        ],
        {"CC": "clang"},
    ),
    Variant(
        "windows-2017-32",
        "Windows (i686) (VS 2017)",
        "windows-vsCurrent-large",
        ["debug-compile-nosasl-nossl", ".latest .nossl .nosasl"],
        {"CC": "Visual Studio 15 2017"},
    ),
    Variant(
        "windows-2017",
        "Windows (VS 2017)",
        "windows-vsCurrent-large",
        [
            "debug-compile-nosasl-nossl",
            "debug-compile-nosasl-openssl",
            "debug-compile-sspi-winssl",
            ".latest .nossl",
            ".nosasl .latest .nossl",
            "test-dns-winssl",
            "test-dns-auth-winssl",
            "debug-compile-aws",
            "test-aws-openssl-regular-4.4",
            "test-aws-openssl-regular-latest",
            # Authentication tests with OpenSSL on Windows are only run on the vs2017 variant.
            # Older vs variants fail to verify certificates against Atlas tests.
            ".authentication-tests .openssl !.sasl",
            ".authentication-tests .winssl",
        ],
        {"CC": "Visual Studio 15 2017 Win64"},
    ),
    Variant(
        "mingw-windows2016",
        "MinGW-W64 (Windows Server 2016)",
        "windows-vsCurrent-large",
        ["debug-compile-nosasl-nossl", ".latest .nossl .nosasl .server"],
        {"CC": "mingw"},
    ),
    Variant("mingw", "MinGW-W64", "windows-vsCurrent-large", ["debug-compile-no-align"], {"CC": "mingw"}),
    Variant(
        "rhel8-power",
        "Power (ppc64le) (RHEL 8)",
        "rhel8-power-large",
        [
            "release-compile",
            "debug-compile-nosasl-nossl",
            "debug-compile-sasl-openssl",
            ".latest .nossl",
            "test-dns-openssl",
        ],
        {"CC": "gcc"},
        patchable=False,
        batchtime=days(1),
    ),
    Variant(
        "arm-ubuntu2004",
        "*ARM (aarch64) (Ubuntu 20.04)",
        "ubuntu2004-arm64-large",
        [
            ".compression !.snappy !.zstd",
            "debug-compile-no-align",
            "release-compile",
            "debug-compile-nosasl-nossl",
            "debug-compile-nosasl-openssl",
            "debug-compile-sasl-openssl",
            ".authentication-tests .openssl",
            ".latest .nossl",
            "test-dns-openssl",
        ],
        {"CC": "gcc"},
        batchtime=days(1),
    ),
    Variant(
        "zseries-rhel8",
        "*zSeries",
        "rhel8-zseries-large",
        [
            "release-compile",
            #      '.compression', --> TODO: waiting on ticket CDRIVER-3258
            "debug-compile-no-align",
            "debug-compile-nosasl-nossl",
            "debug-compile-nosasl-openssl",
            "debug-compile-sasl-openssl",
            ".authentication-tests .openssl",
            ".latest .nossl",
        ],
        {"CC": "gcc"},
        patchable=False,
        batchtime=days(1),
    ),
    Variant(
        "clang100ubuntu",
        "clang 10.0 (Ubuntu 20.04)",
        "ubuntu2004-test",
        [
            "debug-compile-sasl-openssl-static",
            ".authentication-tests .asan",
        ],
        {"CC": "clang"},
    ),
    # Run AWS tests for MongoDB 4.4 and 5.0 on Ubuntu 20.04. AWS setup scripts
    # expect Ubuntu 20.04+. MongoDB 4.4 and 5.0 are not available on 22.04.
    Variant(
        "aws-ubuntu2004",
        "AWS Tests (Ubuntu 20.04)",
        "ubuntu2004-small",
        [
            "debug-compile-aws",
            ".test-aws .4.4",
            ".test-aws .5.0",
        ],
        {"CC": "clang"},
    ),
    Variant(
        "aws-ubuntu2204",
        "AWS Tests (Ubuntu 22.04)",
        "ubuntu2004-small",
        [
            "debug-compile-aws",
            ".test-aws .6.0",
            ".test-aws .7.0",
            ".test-aws .8.0",
            ".test-aws .latest",
        ],
        {"CC": "clang"},
    ),
    Variant("mongohouse", "Mongohouse Test", "ubuntu2204-small", ["debug-compile-sasl-openssl", "test-mongohouse"], {}),
    Variant(
        "ocsp",
        "OCSP tests",
        "ubuntu2004-small",
        [
            OD([("name", "debug-compile-nosasl-openssl")]),
            OD([("name", "debug-compile-nosasl-openssl-static")]),
            OD([("name", "debug-compile-nosasl-darwinssl"), ("distros", ["macos-14-arm64"])]),
            OD([("name", "debug-compile-nosasl-winssl"), ("distros", ["windows-vsCurrent-large"])]),
            OD([("name", ".ocsp-openssl")]),
            OD([("name", ".ocsp-darwinssl"), ("distros", ["macos-14-arm64"])]),
            OD([("name", ".ocsp-winssl"), ("distros", ["windows-vsCurrent-large"])]),
            OD([("name", "debug-compile-nosasl-openssl-1.0.1")]),
            OD([("name", ".ocsp-openssl-1.0.1")]),
        ],
        {},
        batchtime=days(7),
        display_tasks=[
            {
                "name": "ocsp-openssl",
                "execution_tasks": [".ocsp-openssl"],
            },
            {
                "name": "ocsp-darwinssl",
                "execution_tasks": [".ocsp-darwinssl"],
            },
            {
                "name": "ocsp-winssl",
                "execution_tasks": [".ocsp-winssl"],
            },
            {
                "name": "ocsp-openssl-1.0.1",
                "execution_tasks": [".ocsp-openssl-1.0.1"],
            },
        ]
    ),
    Variant(
        "packaging",
        "Linux Distro Packaging",
        "debian12-latest-small",
        [
            "debian-package-build",
            OD([("name", "rpm-package-build"), ("distros", ["rhel90-arm64-small"])]),
        ],
        {},
        tags=["pr-merge-gate"],
    ),
    # Test 7.0+ with Ubuntu 20.04+ since MongoDB 7.0 no longer ships binaries for Ubuntu 18.04.
    Variant(
        "versioned-api-ubuntu2004",
        "Versioned API Tests (Ubuntu 20.04)",
        "ubuntu2004-test",
        [
            "debug-compile-nosasl-openssl",
            "debug-compile-nosasl-nossl",
            ".versioned-api .5.0",
            ".versioned-api .6.0",
            ".versioned-api .7.0",
            ".versioned-api .8.0"
        ],
        {},
    ),
]



from __future__ import annotations

import functools
import re
from typing import Iterable, Literal, Mapping, NamedTuple, TypeVar

from shrub.v3.evg_build_variant import BuildVariant
from shrub.v3.evg_command import BuiltInCommand, EvgCommandType, subprocess_exec
from shrub.v3.evg_task import EvgTask, EvgTaskRef

from ..etc.utils import all_possible

T = TypeVar("T")

_ENV_PARAM_NAME = "MONGOC_EARTHLY_ENV"
_CC_PARAM_NAME = "MONGOC_EARTHLY_C_COMPILER"
"The name of the EVG expansion for the Earthly c_compiler argument"


EnvKey = Literal[
    "u16",
    "u18",
    "u20",
    "u22",
    "alpine3.16",
    "alpine3.17",
    "alpine3.18",
    "alpine3.19",
    "archlinux",
    "centos7"
]
"Identifiers for environments. These correspond to special 'env.*' targets in the Earthfile."
CompilerName = Literal["gcc", "clang"]
"The name of the compiler program that is used for the build. Passed via --c_compiler to Earthly."

# Other options: SSPI (Windows only), AUTO (not reliably test-able without more environments)
SASLOption = Literal["Cyrus", "off"]
"Valid options for the SASL configuration parameter"
TLSOption = Literal["LibreSSL", "OpenSSL", "off"]
"Options for the TLS backend configuration parameter (AKA 'ENABLE_SSL')"
CxxVersion = Literal["r3.8.0", "r3.9.0", "none"]
"C++ driver refs that are under CI test"

# A separator character, since we cannot use whitespace
_SEPARATOR = "\N{no-break space}\N{bullet}\N{no-break space}"


def os_split(env: EnvKey) -> tuple[str, None | str]:
    """Convert the environment key into a pretty name+version pair"""
    match env:
        # match 'alpine3.18' 'alpine53.123' etc.
        case alp if mat := re.match(r"alpine(\d+\.\d+)", alp):
            return ("Alpine", mat[1])
        case "archlinux":
            return "ArchLinux", None
        # Match 'u22', 'u20', 'u71' etc.
        case ubu if mat := re.match(r"u(\d\d)", ubu):
            return "Ubuntu", f"{mat[1]}.04"
        case "centos7":
            return "CentOS", "7.0"
        case _:
            raise ValueError(
                f"Failed to split OS env key {env=} into a name+version pair (unrecognized)"
            )


class EarthlyVariant(NamedTuple):
    """
    Define a "variant" that runs under a set of Earthly parameters. These are
    turned into real EVG variants later on. The Earthly arguments are passed via
    expansion parameters.
    """

    env: EnvKey
    c_compiler: CompilerName

    @property
    def display_name(self) -> str:
        """The pretty name for this variant"""
        base: str
        match os_split(self.env):
            case name, None:
                base = name
            case name, version:
                base = f"{name} {version}"
        toolchain: str
        match self.c_compiler:
            case "clang":
                toolchain = "LLVM/Clang"
            case "gcc":
                toolchain = "GCC"
        return f"{base} ({toolchain})"

    @property
    def task_selector_tag(self) -> str:
        """
        The task tag that is used to select the tasks that want to run on this
        variant.
        """
        return f"{self.env}-{self.c_compiler}"

    @property
    def expansions(self) -> Mapping[str, str]:
        """
        Expansion values that are defined for the build variant that is generated
        from this object.
        """
        return {
            _CC_PARAM_NAME: self.c_compiler,
            _ENV_PARAM_NAME: self.env,
        }

    def as_evg_variant(self) -> BuildVariant:
        return BuildVariant(
            name=f"{self.task_selector_tag}",
            tasks=[EvgTaskRef(name=f".{self.task_selector_tag}")],
            display_name=self.display_name,
            expansions=dict(self.expansions),
        )


class Configuration(NamedTuple):
    """
    Represent a complete set of argument values to give to the Earthly task
    execution. Each field name matches the ARG in the Earthfile.

    Adding/removing fields will add/remove dimensions on the task matrix.

    Some Earthly parameters are not encoded here, but are rather part of the variant (EarthlyVariant).
    """

    sasl: SASLOption
    tls: TLSOption
    test_mongocxx_ref: CxxVersion

    @property
    def suffix(self) -> str:
        return f"{_SEPARATOR}".join(f"{k}={v}" for k, v in self._asdict().items())


def task_filter(env: EarthlyVariant, conf: Configuration) -> bool:
    """
    Control which tasks are actually defined by matching on the platform and
    configuration values.
    """
    match env, conf:
        # We only need one task with "sasl=off"
        case ["u22", "gcc"], ("off", "OpenSSL", "r3.8.0"):
            return True
        # The Ubuntu 18.04 GCC has a bug that fails to build the 3.8.0 C++ driver
        case ["u18", "gcc"], [_, _, "r3.8.0"]:
            return False
        # Other sasl=off tasks we'll just ignore:
        case _, ("off", _tls, _cxx):
            return False
        # Ubuntu and CentOS do not ship with a LibreSSL package:
        case e, (_sasl, "LibreSSL", _cxx) if e.display_name.startswith("Ubuntu") or e.display_name.startswith("CentOS"):
            return False
        # u16 is not capable of building mongocxx
        case e, (_, _, "none") if e.display_name.startswith("Ubuntu 16") or e.display_name.startswith("CentOS 7"):
            return True
         # Exclude u16 for all other configurations
        case e, _ if e.display_name.startswith("Ubuntu 16") or e.display_name.startswith("CentOS 7"):
            return False
        # Exclude all other envs
        case _, (_, _, "none"):
            return False
        # Anything else: Allow it to run:
        case _:
            return True


def variants_for(config: Configuration) -> Iterable[EarthlyVariant]:
    """Get all Earthly variants that are not excluded for the given build configuration"""
    all_envs = all_possible(EarthlyVariant)
    allow_env_for_config = functools.partial(task_filter, conf=config)
    return filter(allow_env_for_config, all_envs)


def earthly_exec(
    *,
    kind: Literal["test", "setup", "system"],
    target: str,
    secrets: Mapping[str, str] | None = None,
    args: Mapping[str, str] | None = None,
) -> BuiltInCommand:
    """Create a subprocess_exec command that runs Earthly with the given arguments"""
    env: dict[str, str] = {k: v for k, v in (secrets or {}).items()}
    return subprocess_exec(
        "./tools/earthly.sh",
        args=[
            *(f"--secret={k}" for k in (secrets or ())),
            f"+{target}",
            *(f"--{arg}={val}" for arg, val in (args or {}).items()),
        ],
        command_type=EvgCommandType(kind),
        env=env if env else None,
        working_dir="mongoc",
    )


def earthly_task(
    *,
    name: str,
    targets: Iterable[str],
    config: Configuration,
) -> EvgTask | None:
    """
    Create an EVG task which executes earthly using the given parameters. If this
    function returns `None`, then the task configuration is excluded from executing
    and no task should be defined.
    """
    # Attach tags to the task to allow build variants to select
    # these tasks by the environment of that variant.
    env_tags = sorted(e.task_selector_tag for e in sorted(variants_for(config)))
    if not env_tags:
        # All environments have been excluded for this configuration. This means
        # the task itself should not be run:
        return
    # Generate the build-arg arguments based on the configuration options. The
    # NamedTuple field names must match with the ARG keys in the Earthfile!
    earthly_args = config._asdict()
    earthly_args |= {
        # Add arguments that come from parameter expansions defined in the build variant
        "env": f"${{{_ENV_PARAM_NAME}}}",
        "c_compiler": f"${{{_CC_PARAM_NAME}}}",
    }
    return EvgTask(
        name=name,
        commands=[
            # Ensure subsequent Docker commands are authenticated.
            subprocess_exec(
                binary='bash',
                command_type="setup",
                args=[
                    "-c",
                    'docker login -u "${artifactory_username}" --password-stdin artifactory.corp.mongodb.com <<<"${artifactory_password}"'
                ],
            ),
            # First, just build the "env-warmup" which will prepare the build environment.
            # This won't generate any output, but allows EVG to track it as a separate build step
            # for timing and logging purposes. The subequent build step will cache-hit the
            # warmed-up build environments.
            earthly_exec(
                kind="setup",
                target="env-warmup",
                args=earthly_args,
            ),
            # Now execute the main tasks:
            earthly_exec(
                kind="test",
                target="run",
                # The "targets" arg is for +run to specify which targets to run
                args={"targets": " ".join(targets)} | earthly_args,
            ),
        ],
        tags=[f"earthly", "pr-merge-gate", *env_tags],
        run_on=CONTAINER_RUN_DISTROS,
    )


CONTAINER_RUN_DISTROS = [
    "ubuntu2204-large",
    "debian10-large",
    "debian11-large",
    "amazon2",
]


def tasks() -> Iterable[EvgTask]:
    for conf in all_possible(Configuration):
        # test-example is a target in all configurations
        targets = ["test-example"]

        # test-cxx-driver is only a target in configurations with specified mongocxx versions
        if conf.test_mongocxx_ref != "none":
            targets.append("test-cxx-driver")

        task = earthly_task(
            name=f"check:{conf.suffix}",
            targets=targets,
            config=conf,
        )
        if task is not None:
            yield task


def variants() -> Iterable[BuildVariant]:
    yield from (ev.as_evg_variant() for ev in all_possible(EarthlyVariant))



#!/usr/bin/env python
#
# Copyright 2009-present MongoDB, Inc.
#
# Licensed under the Apache License, Version 2.0(the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http: // www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from collections import OrderedDict as OD
from typing import MutableSequence

from config_generator.components.funcs.find_cmake_latest import FindCMakeLatest

from evergreen_config_generator.functions import shell_exec, func
from evergreen_config_generator.tasks import NamedTask
from evergreen_config_generator.variants import Variant
from evergreen_config_generator.taskgroups import TaskGroup


def _create_tasks():
    passtask = NamedTask(
        task_name="testgcpkms-task",
        commands=[
            func("fetch-source"),
            func("find-cmake-latest"),
            shell_exec(
                r"""
            echo "Building test-gcpkms ... begin"
            pushd mongoc
            .evergreen/scripts/compile-test-gcpkms.sh
            popd
            echo "Building test-gcpkms ... end"
            echo "Copying files ... begin"
            export GCPKMS_GCLOUD=${GCPKMS_GCLOUD}
            export GCPKMS_PROJECT=${GCPKMS_PROJECT}
            export GCPKMS_ZONE=${GCPKMS_ZONE}
            export GCPKMS_INSTANCENAME=${GCPKMS_INSTANCENAME}
            DRIVERS_TOOLS=$(pwd)/drivers-evergreen-tools
            mkdir testgcpkms
            cp ./mongoc/src/libmongoc/test-gcpkms ./mongoc/install/lib/libmongocrypt.* testgcpkms
            tar czf testgcpkms.tgz testgcpkms/*
            GCPKMS_SRC="testgcpkms.tgz" GCPKMS_DST=$GCPKMS_INSTANCENAME: $DRIVERS_TOOLS/.evergreen/csfle/gcpkms/copy-file.sh
            echo "Copying files ... end"
            echo "Untarring file ... begin"
            GCPKMS_CMD="tar xf testgcpkms.tgz" $DRIVERS_TOOLS/.evergreen/csfle/gcpkms/run-command.sh
            echo "Untarring file ... end"
            """,
                test=False,
                add_expansions_to_env=True,
            ),
            shell_exec(
                r"""
            export GCPKMS_GCLOUD=${GCPKMS_GCLOUD}
            export GCPKMS_PROJECT=${GCPKMS_PROJECT}
            export GCPKMS_ZONE=${GCPKMS_ZONE}
            export GCPKMS_INSTANCENAME=${GCPKMS_INSTANCENAME}
            DRIVERS_TOOLS=$(pwd)/drivers-evergreen-tools
            GCPKMS_CMD="LD_LIBRARY_PATH=./testgcpkms MONGODB_URI='mongodb://localhost:27017' ./testgcpkms/test-gcpkms" $DRIVERS_TOOLS/.evergreen/csfle/gcpkms/run-command.sh
            """
            ),
        ],
    )

    failtask = NamedTask(
        task_name="testgcpkms-fail-task",
        commands=[
            func("find-cmake-latest"),
            shell_exec(
                r"""
            pushd mongoc
            .evergreen/scripts/compile-test-gcpkms.sh
            popd""",
                test=False,
                add_expansions_to_env=True,
            ),
            shell_exec(
                r"""
            export GCPKMS_GCLOUD=${GCPKMS_GCLOUD}
            export GCPKMS_PROJECT=${GCPKMS_PROJECT}
            export GCPKMS_ZONE=${GCPKMS_ZONE}
            export GCPKMS_INSTANCENAME=${GCPKMS_INSTANCENAME}
            LD_LIBRARY_PATH=$(pwd)/install MONGODB_URI='mongodb://localhost:27017' EXPECT_ERROR='Failed to connect to: metadata.google.internal' ./mongoc/src/libmongoc/test-gcpkms"""
            ),
        ],
    )

    return [passtask, failtask]


def _create_variant():
    return Variant(
        name="testgcpkms-variant",
        display_name="GCP KMS",
        # GCP Virtual Machine created is Debian 11.
        run_on="debian11-small",
        tasks=["testgcpkms_task_group", "testgcpkms-fail-task"],
        batchtime=20160,
    )  # Use a batchtime of 14 days as suggested by the CSFLE test README


def _create_task_group():
    task_group = TaskGroup(name="testgcpkms_task_group")
    task_group.setup_group_can_fail_task = True
    task_group.setup_group_timeout_secs = 1800  # 30 minutes
    task_group.setup_group = [
        func("fetch-det"),
        # Create and set up a GCE instance using driver tools script
        shell_exec(
            r"""
            DRIVERS_TOOLS=$(pwd)/drivers-evergreen-tools
            echo '${testgcpkms_key_file}' > /tmp/testgcpkms_key_file.json
            export GCPKMS_KEYFILE=/tmp/testgcpkms_key_file.json
            export GCPKMS_DRIVERS_TOOLS=$DRIVERS_TOOLS
            export GCPKMS_SERVICEACCOUNT="${testgcpkms_service_account}"
            $DRIVERS_TOOLS/.evergreen/csfle/gcpkms/create-and-setup-instance.sh""",
            test=False,
        ),
        # Load the GCPKMS_GCLOUD, GCPKMS_INSTANCE, GCPKMS_PROJECT, and GCPKMS_ZONE expansions.
        OD([("command", "expansions.update"), ("params", OD([("file", "testgcpkms-expansions.yml")]))]),
    ]

    task_group.teardown_group = [
        shell_exec(
            r"""
            DRIVERS_TOOLS=$(pwd)/drivers-evergreen-tools
            export GCPKMS_GCLOUD=${GCPKMS_GCLOUD}
            export GCPKMS_PROJECT=${GCPKMS_PROJECT}
            export GCPKMS_ZONE=${GCPKMS_ZONE}
            export GCPKMS_INSTANCENAME=${GCPKMS_INSTANCENAME}
            $DRIVERS_TOOLS/.evergreen/csfle/gcpkms/delete-instance.sh""",
            test=False,
        )
    ]
    task_group.tasks = ["testgcpkms-task"]
    return task_group


def testgcpkms_generate(
    all_tasks: MutableSequence[NamedTask],
    all_variants: MutableSequence[Variant],
    all_task_groups: MutableSequence[TaskGroup],
):
    all_tasks.extend(_create_tasks())
    all_variants.append(_create_variant())
    all_task_groups.append(_create_task_group())



from config_generator.etc.compile import generate_compile_tasks

from config_generator.etc.sanitizers.test import generate_test_tasks

from config_generator.components.cse.openssl import SaslCyrusOpenSSLCompile

from config_generator.components.sanitizers.asan import TAG


# pylint: disable=line-too-long
# fmt: off
COMPILE_MATRIX = [
    ('ubuntu2004', 'clang', None, ['cyrus']),
    ('debian10',   'clang', None, ['cyrus']),
]

TEST_MATRIX = [
    # Test 7.0+ with a replica set since Queryable Encryption does not support the 'server' topology. Queryable Encryption tests require 7.0+.
    ('ubuntu2004', 'clang', None, 'cyrus', ['auth'], ['server', 'replica'], ['4.4', '5.0', '6.0', '7.0', '8.0', 'latest']),

    # Test 4.2 with Debian 10 since 4.2 does not ship on Ubuntu 20.04+.
    ('debian10',   'clang', None, 'cyrus', ['auth'], ['server'], ['4.2']), 
]
# fmt: on
# pylint: enable=line-too-long


MORE_TAGS = ['cse', 'asan']


def tasks():
    res = []

    SSL = 'openssl'
    SASL_TO_FUNC = {
        'cyrus': SaslCyrusOpenSSLCompile,
    }

    res += generate_compile_tasks(
        SSL, TAG, SASL_TO_FUNC, COMPILE_MATRIX, MORE_TAGS
    )

    res += generate_test_tasks(SSL, TAG, TEST_MATRIX, MORE_TAGS)

    res += generate_test_tasks(
        SSL, TAG, TEST_MATRIX, MORE_TAGS,
        MORE_TEST_TAGS=['with-mongocrypt'],
        MORE_VARS={'SKIP_CRYPT_SHARED_LIB': 'on'}
    )

    return res



from shrub.v3.evg_command import EvgCommandType
from shrub.v3.evg_command import s3_put
from shrub.v3.evg_task import EvgTask

from config_generator.etc.function import Function
from config_generator.etc.utils import bash_exec


class KmsDivergenceCheck(Function):
    name = "kms-divergence-check"
    commands = [
        bash_exec(
            command_type=EvgCommandType.TEST,
            working_dir="mongoc",
            script=".evergreen/scripts/kms-divergence-check.sh"
        ),
    ]

    @classmethod
    def call(cls, **kwargs):
        return cls.default_call(**kwargs)


def functions():
    return KmsDivergenceCheck.defn()


def tasks():
    return [
        EvgTask(
            name=KmsDivergenceCheck.name,
            commands=[
                KmsDivergenceCheck.call(),
            ],
        )
    ]



from shrub.v3.evg_build_variant import BuildVariant
from shrub.v3.evg_task import EvgTaskRef


TAG = 'sanitizers-matrix-asan'


def variants():
    expansions = {
        'ASAN': 'on',
        'CFLAGS': '-fno-omit-frame-pointer',
        'CHECK_LOG': 'ON',
        'EXTRA_CONFIGURE_FLAGS': '-DENABLE_EXTRA_ALIGNMENT=OFF',
        'SANITIZE': 'address,undefined',
    }

    return [
        BuildVariant(
            name=TAG,
            display_name=TAG,
            tasks=[EvgTaskRef(name=f'.{TAG}')],
            expansions=expansions,
        ),
    ]



from config_generator.etc.function import Function
from config_generator.etc.utils import bash_exec


class RestoreInstanceProfile(Function):
    name = 'restore-instance-profile'
    commands = [
        bash_exec(
            script='''\
            # Restore the AWS Instance Profile that may have been removed in AWS tasks.

            if [[ ! -d drivers-evergreen-tools ]]; then
                echo "drivers-evergreen-tools not present ... skipping"
                exit 0
            fi

            cd drivers-evergreen-tools/.evergreen/auth_aws
            if [[ ! -f aws_e2e_setup.json ]]; then
                echo "aws_e2e_setup.json not present ... skipping"
                exit 0
            fi

            . ./activate-authawsvenv.sh
            
            echo "restoring instance profile ... "
            # Capture and hide logs on success. Logs may included expected `HTTP Error 404: Not Found` messages when checking for instance profile.
            if ! { python ./lib/aws_assign_instance_profile.py 2>&1 >|output.txt; }; then
                echo "restoring instance profile ... failed"
                cat output.txt 1>&2
                exit 1
            fi
            echo "restoring instance profile ... succeeded"
            '''
        ),
    ]

    @classmethod
    def call(cls, **kwargs):
        return cls.default_call(**kwargs)


def functions():
    return RestoreInstanceProfile.defn()



# Copyright 2009-present MongoDB, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import Iterable
from evergreen_config_generator.functions import shell_exec


def shell_mongoc(
    script: str,
    test: bool = True,
    errexit: bool = True,
    xtrace: bool = False,
    silent: bool = False,
    continue_on_err: bool = False,
    background: bool = False,
    add_expansions_to_env: bool = False,
    redirect_standard_error_to_output: bool = False,
    include_expansions_in_env: Iterable[str] = (),
):
    return shell_exec(
        script,
        working_dir="mongoc",
        test=test,
        errexit=errexit,
        xtrace=xtrace,
        silent=silent,
        continue_on_err=continue_on_err,
        background=background,
        add_expansions_to_env=add_expansions_to_env,
        include_expansions_in_env=include_expansions_in_env,
        redirect_standard_error_to_output=redirect_standard_error_to_output,
    )



from shrub.v3.evg_build_variant import BuildVariant
from shrub.v3.evg_command import EvgCommandType, FunctionCall, expansions_update
from shrub.v3.evg_task import EvgTask, EvgTaskRef, EvgTaskDependency

from config_generator.components.funcs.bootstrap_mongo_orchestration import BootstrapMongoOrchestration
from config_generator.components.funcs.fetch_build import FetchBuild
from config_generator.components.funcs.fetch_det import FetchDET
from config_generator.components.funcs.find_cmake_latest import FindCMakeLatest
from config_generator.components.funcs.run_simple_http_server import RunSimpleHTTPServer
from config_generator.components.funcs.run_tests import RunTests
from config_generator.components.funcs.upload_build import UploadBuild
from config_generator.etc.distros import make_distro_str, find_small_distro, find_large_distro
from config_generator.etc.utils import bash_exec

# Use `rhel8.9` distro. `rhel8.9` distro includes necessary dependency: `haproxy`.
_DISTRO_NAME = "rhel8.9"
_COMPILER = "gcc"


def functions():
    return {
        'start-load-balancer': [
            bash_exec(
                command_type=EvgCommandType.SETUP,
                script='''\
                    export DRIVERS_TOOLS=./drivers-evergreen-tools
                    export MONGODB_URI="${MONGODB_URI}"
                    $DRIVERS_TOOLS/.evergreen/run-load-balancer.sh start
                ''',
            ),
            expansions_update(
                command_type=EvgCommandType.SETUP,
                file='lb-expansion.yml',
            )
        ]
    }


def make_test_task(auth: bool, ssl: bool, server_version: str):
    auth_str = "auth" if auth else "noauth"
    ssl_str = "openssl" if ssl else "nossl"
    distro_str = make_distro_str(_DISTRO_NAME, _COMPILER, None)
    return EvgTask(
        name=f"loadbalanced-{distro_str}-test-{server_version}-{auth_str}-{ssl_str}",
        depends_on=[EvgTaskDependency(
            name=f"loadbalanced-{distro_str}-compile")],
        run_on=find_small_distro(_DISTRO_NAME).name,
        tags=['loadbalanced', _DISTRO_NAME, _COMPILER, auth_str, ssl_str],
        commands=[
            FetchBuild.call(build_name=f"loadbalanced-{distro_str}-compile"),
            FetchDET.call(),
            BootstrapMongoOrchestration().call(vars={
                'AUTH': auth_str,
                'SSL': ssl_str,
                'MONGODB_VERSION': server_version,
                'TOPOLOGY': 'sharded_cluster',
                'LOAD_BALANCER': 'on',
            }),
            RunSimpleHTTPServer.call(),
            FunctionCall(func='start-load-balancer', vars={
                'MONGODB_URI': 'mongodb://localhost:27017,localhost:27018'
            }),
            RunTests().call(vars={
                'AUTH': auth_str,
                'SSL': ssl_str,
                'LOADBALANCED': 'loadbalanced',
                'CC': _COMPILER,
            })
        ],
    )


def tasks():
    distro_str = make_distro_str(_DISTRO_NAME, _COMPILER, None)
    yield EvgTask(
        name=f"loadbalanced-{distro_str}-compile",
        run_on=find_large_distro(_DISTRO_NAME).name,
        tags=['loadbalanced', _DISTRO_NAME, _COMPILER],
        commands=[
            FindCMakeLatest.call(),
            bash_exec(
                command_type=EvgCommandType.TEST,
                env={
                    'CC': _COMPILER,
                    'CFLAGS': '-fno-omit-frame-pointer',
                    'EXTRA_CONFIGURE_FLAGS': '-DENABLE_EXTRA_ALIGNMENT=OFF',
                    'SSL': 'OPENSSL'
                },
                working_dir='mongoc',
                script='.evergreen/scripts/compile.sh',
            ),
            UploadBuild.call()
        ],
    )

    # Satisfy requirements specified in
    # https://github.com/mongodb/specifications/blob/master/source/load-balancers/tests/README.md#testing-requirements
    #
    # > For each server version that supports load balanced clusters, drivers
    # > MUST add two Evergreen tasks: one with a sharded cluster with both
    # > authentication and TLS enabled and one with a sharded cluster with
    # > authentication and TLS disabled.
    server_versions = ['5.0', '6.0', '7.0', '8.0', 'latest']
    for server_version in server_versions:
        yield make_test_task(auth=False, ssl=False, server_version=server_version)
        yield make_test_task(auth=True, ssl=True, server_version=server_version)


def variants():
    return [
        BuildVariant(
            name="loadbalanced",
            display_name="loadbalanced",
            tasks=[EvgTaskRef(name='.loadbalanced')]
        ),
    ]



#!/usr/bin/env python
#
# Copyright 2009-present MongoDB, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Generate C Driver's config.yml for Evergreen testing.

We find that generating configuration from Python data structures is more
legible than Evergreen's matrix syntax or a handwritten file.

Requires the evergreen_config_generator package.
"""

from collections import OrderedDict as OD
from os.path import dirname, join as joinpath, normpath

from evergreen_config_generator import generate

from evergreen_config_lib.functions import all_functions
from evergreen_config_lib.tasks import all_tasks
from evergreen_config_lib.variants import all_variants
from evergreen_config_lib.taskgroups import all_task_groups
from evergreen_config_lib.testgcpkms import testgcpkms_generate
from evergreen_config_lib.testazurekms import testazurekms_generate

task_groups = list(all_task_groups)
testazurekms_generate(all_tasks, all_variants, task_groups)
testgcpkms_generate(all_tasks, all_variants, task_groups)

config = OD(
    [
        ("functions", all_functions),
        ("tasks", all_tasks),
        ("task_groups", task_groups),
        ("buildvariants", all_variants),
    ]
)

this_dir = dirname(__file__)
generated_configs_dir = normpath(joinpath(this_dir, "../generated_configs"))
generate(config, joinpath(generated_configs_dir, "legacy-config.yml"))



from typing import ClassVar

from shrub.v3.evg_command import EvgCommand
from shrub.v3.evg_command import EvgCommandType

from config_generator.etc.utils import bash_exec

from config_generator.etc.function import Function


class CompileCommon(Function):
    ssl: ClassVar[str | None]

    @classmethod
    def compile_commands(cls, sasl=None) -> list[EvgCommand]:
        env = {}

        if cls.ssl:
            env['SSL'] = cls.ssl

        if sasl:
            env['SASL'] = sasl

        return [
            bash_exec(
                command_type=EvgCommandType.TEST,
                add_expansions_to_env=True,
                env=env,
                working_dir='mongoc',
                script='.evergreen/scripts/compile.sh',
            ),
        ]

    @classmethod
    def call(cls, **kwargs):
        return cls.default_call(**kwargs)
